:userdoc.
:title.Universal Language Support Programming Guide

.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=10000.Universal Language Support
:p.The :hp2.Universal Language Support:ehp2. API (ULS) is designed to facilitate
the development of internationalized applications in conjunction with the
Unicode standard.

:p.The ULS functions fall into four major categories&colon.
:dl tsize=28 break=fit.
:dt.:link reftype=hd res=20000.Unicode Text Processing:elink.
:dd.Provides functions for working with multilingual Unicode (UCS-2) text.
:dt.:link reftype=hd res=40000.Localization:elink.
:dd.Deals with the locale-specific presentation of information, and other
conventions that may vary according to country or region.
:dt.:link reftype=hd res=30000.Codepage Conversion:elink.
:dd.Allows text to be converted to and from Unicode or from one codepage
to another.
:dt.Keyboard Input
:dd.Allows the conversion of keyboard scan codes to and from Unicode characters
according to different keyboard layouts.  Not documented here.
:edl.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=10100.Programming with ULS
:p.Programs written to use ULS must include the appropriate header files&colon.
:dl compact tsize=14.
:dt.:hp2.unidef.h:ehp2.
:dd.Main ULS API definitions
:dt.:hp2.uconv.h:ehp2.
:dd.ULS conversion API definitions
:dt.:hp2.unikbd.h:ehp2.
:dd.ULS keyboard API definitions
:edl.

:p.The corresponding import libraries should be linked at build time&colon.
:dl compact tsize=14.
:dt.:hp2.libuls.lib:ehp2.
:dd.ULS text processing and localization functions
:dt.:hp2.libconv.lib:ehp2.
:dd.ULS conversion functions
:dt.:hp2.unikbd.lib:ehp2.
:dd.ULS keyboard functions
:edl.


.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=20000.Unicode Text Processing
:p.:hp2.Unicode:ehp2. is a character standard designed to allow all possible
characters from all known writing systems to be represented on computer systems
in an interchangeable form.

:p.Under Unicode, every character is assigned a unique numeric value.  In this
respect, Unicode is similar to ASCII, EBCDIC and other common character
standards used on various computer platforms.  What distinguishes Unicode is
its scope&colon. this single character standard is capable of consistently and
unambiguously representing tens of thousands of unique characters from a vast
range of different languages and writing systems.

:p.The mapping of characters to numeric values defined by Unicode is also
known as the :hp2.Universal Character Set:ehp2. (:hp2.UCS:ehp2.).

:nt.Besides UCS, the Unicode standard also defines various algorithms and other
implementation requirements.  In practice, however, the terms "UCS" and
"Unicode" are often used interchangeably.:ent.

:p.In order to maximize compatibility with legacy ASCII-based encodings, UCS
values 0x20 through 0x7F correspond to the displayable ASCII character set.  It
is therefore easy to convert these characters to and from Unicode; the specifics
of conversion depend on the precise Unicode encoding used.

:nt.When describing Unicode values, they are traditionally written in the
form :hp2.U+:ehp2.:hp3.x:ehp3., where :hp1.x:ehp1. is the character's
hexadecimal UCS codepoint.:ent.

:p.Version 1 of the Unicode standard defined a range of 65,536 (2^16) character
values (or codepoints), intended to cover most of the world's modern writing
systems as well as many other common symbols.

:p.The ultimate goal of Unicode, however, is to encompass support for all forms
of text, including ancient languages, mathematical and musical symbols, and rare
or obscure characters.  To this end, the Unicode standard was expanded (as of
version 2.0) to encompass 17 :hp2.planes:ehp2., each of which contains 65,536
codepoints (for a total codespace of 1,114,112 values, or U+0000 through
U+10FFFF).

:p.The 65,536 codepoints defined by the original standard (U+0000 - U+FFFF) are
now known as Plane 0 or the :hp2.Basic Multilingual Plane:ehp2.
(:hp2.BMP:ehp2.).  As of the latest versions of Unicode, the character sets
supported by the BMP include&colon.

:ul compact.
:li.Arabic
:li.Armenian
:li.Braille patterns
:li.Burmese
:li.Canadian Aboriginal unified syllabary
:li.Cherokee
:li.Chinese, Japanese and Korean (CJK) unified ideographic characters
:li.Chinese scripts and symbols
:li.Coptic
:li.Cyrillic
:li.Diacritics and modifying characters
:li.Ethiopic
:li.Filipino scripts (Tagalog, Hanunoo, Buhid, and Tagbanwa)
:li.Georgian
:li.Greek
:li.Hebrew
:li.Indic scripts (Thaana, Devanagari, Bengali, Gurmukhi, Gujarati, Oriya, Tamil,
Telugu, Kannada, Malayalam, and Sinhala)
:li.International Phonetic Alphabet (IPA)
:li.Japanese scripts and symbols
:li.Khmer
:li.Korean scripts and symbols
:li.Lao
:li.Latin (including Western European, Slavic, Baltic, Nordic, and Turkish)
:li.Limbu
:li.Mongolian
:li.Ogham
:li.Runic
:li.Symbols (punctuation, currency symbols, number forms, mathematical,
notational and technical symbols, dingbats, arrows, shapes, and drawing
characters)
:li.Syriac
:li.Thai
:li.Tibetan
:eul.

:p.Planes 1 and 2 contain mostly obscure, historical, and/or specialized
notational characters; most of the remaining planes are currently
unassigned.  See :link reftype=hd res=50000.Appendix A:elink. for more
information.

:p.At present, OS/2 supports only the Basic Multilingual Plane; otherwise,
the Unicode version supported by the ULS API is approximately equivalent to
Unicode 2.1; this is principally important with respect to the codepage
conversion, text transformation, and character classification functions.  Appendix
A includes a list of :link reftype=hd res=50100.character sets supported by
version 2:elink. of Unicode.

:p.Actual rendering of characters defined by later versions of the Unicode
standard should be possible, as this support depends on the specific font(s)
used, and not the ULS APIs themselves.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=20100.UCS-2 and the UniChar type
:p.UCS text can be encoded in a number of different ways.  The most basic
method is to represent each character as a two-byte integer equal to its
assigned UCS value.  This form of encoding is known as :hp2.UCS-2:ehp2., and
is what OS/2 Universal Language Support uses for Unicode text processing.

:p.Using UCS-2, up to 65,536 unique characters may be represented.  This range
corresponds to the UCS Basic Multilingual Plane.

:nt.Unicode values beyond this range cannot be represented using UCS-2
encoding.  Although this effectively limits OS/2 to the 65,536 characters
within the BMP, this should be sufficient for comprehensive character support
for almost any modern language.:ent.

:p.ULS represents a Unicode character using the :hp2.UniChar:ehp2. data type,
which is a two-byte integer value&colon.
:xmp.
    typedef unsigned short UniChar;
:exmp.
:p.A UniChar is also sometimes referred to as a "UCS code element".


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20101.Other Unicode encodings
:p.There are several other common encodings for Unicode text besides
UCS-2.  The encoding which is used in any particular context normally depends
on a number of factors, including&colon.
:ul compact.
:li.The need for backwards compatibility with older encoding formats and/or APIs.
:li.Size and data storage considerations.
:li.Processing overhead.
:li.Whether the current environment guarantees eight-bit character integrity.
:eul.

:p.Unicode encodings are either fixed-width (in which each character is
represented by a fixed number of bytes), or variable-width (in which the number
of bytes required to represent any given character varies depending on the
character's value).

:p.The most common Unicode encoding formats are listed below.  Not all of these
are supported by the ULS API; unsupported encodings are indicated by an asterisk
(*).

:p.
:p.:hp7.Common Unicode Encodings:ehp7.
:dl break=fit.
:dt.UCS-2
:dd.The oldest and simplest Unicode encoding.  Each character is represented
by a fixed-width two-byte integer value corresponding to its UCS codepoint
within the Basic Multilingual Plane.  Other planes beyond the BMP are not
supported.  Under OS/2, UCS-2 is used for internal processing, but should
generally be avoided for text output.
:p.The main advantage of UCS-2 is its simplicity&colon. a UCS-2 value
corresponds directly to its character's UCS codepoint, with no conversion
needed.  It also requires no more than two bytes to represent any character
within the BMP.

:dt.UTF-32 *
:dd.Originally known as UCS-4, this is essentially a double-width UCS-2.  Each
character is represented by a fixed-width four-byte integer value corresponding
to its UCS codepoint.  Unlike UCS-2, the entire Unicode codespace is supported.
However, because each character requires four bytes of storage, it is quite
inefficient; consequently, it is rarely used.
:p.UTF-32 is not supported by ULS.

:dt.UTF-16 *
:dd.A variable-width encoding designed to be backwards-compatible with
UCS-2.  Each Unicode character within the Basic Multilingual Plane is encoded
using two bytes, and is identical to the equivalent UCS-2 value.  Every
character outside the BMP is encoded using four bytes, in which the lead byte
of each pair is a special "surrogate" value from a range unused within the BMP
(U+D800 through U+DFFF are reserved for this purpose).
:p.The main advantage of UTF-16 is that it is backwards-compatible with
UCS-2&colon. any valid UCS-2 character is also a valid UTF-16 character.  It
thus retains most of UCS-2's advantages, while adding the ability to encode
Unicode values beyond the BMP.
:p.UTF-16 is not supported by ULS.

:dt.UTF-8
:dd.A variable-width encoding designed primarily for data interchange.  Between
one and four bytes are used to represent each character, depending on its
value.  Character values of U+007F and below (basic ASCII) are represented by a
single byte, values from U+0080 to U+07FF are represented by two bytes, values
from U+0800 to U+FFFF are represented by three bytes, and values of U+10000 and
higher are represented by four bytes.  A single-byte character always starts
with a 0 bit.  For multi-byte characters, the number of bytes used is indicated
by the number of leading 1 bits (two or more) at the start of the first byte; a
following byte always starts with the bit sequence 10.
:p.UTF-8 has several important advantages.  First of all, it is backwards
compatible with single-byte ASCII; that is, all ASCII text is also valid UTF-8
text, which makes it ideal for data interchange.  In addition, while UTF-8 is
capable of representing the entire Unicode codespace, it requires considerably
less storage than UTF-32; it may require slightly less than UCS-2 and UTF-16 as
well, although this depends on the text being encoded.  Finally, every byte
in a UTF-8 data stream is instantly identifiable as a lead byte, a following
byte, or a single-byte character&colon. this means that a lost or corrupted byte
can only compromise the current character, and not subsequent data.
:p.The major disadvantage of UTF-8 is its overhead&colon. it is significantly
more complex than UTF-16, which may impact performance under some circumstances.
Consequently, UTF-8 is generally used for output rather than internal
processing.

:dt.UTF-7 *
:dd.A variable-width encoding which only uses seven bits per byte, UTF-7 is
designed for environments when eight-bit integrity is not guaranteed (such as
Internet e-mail systems).  UTF-7 allows most standard ASCII characters to be
encoded verbatim; other characters are encoded using UTF-16 and then re-encoded
using a modified Base64 algorithm.
:p.UTF-7 is rarely used, as it is complex and awkward to implement.  Standard
MIME encoding or quoted-printable UTF-8 are generally preferred in environments
where eight-bit integrity is not guaranteed.
:p.UTF-7 is not supported by ULS.

:dt.UPF-8
:dd.This is a private variable-width encoding specific to OS/2.  Known as
Unicode Processing Format, it is designed to allow Unicode text output under
Presentation Manager (which is not generally capable of rendering UCS-2 text
directly).  UPF-8 requires from one to three bytes to represent a single
character, but has considerably less processing overhead than UTF-8.
:p.Because it is an OS/2-specific encoding, UPF-8 should not be used for data
interchange.
:edl.

:p.
:p.:hp7.Encoding Comparison:ehp7.
:p.UCS-2 was the standard encoding used in the days before Unicode was expanded
beyond the Basic Multilingual Plane.  UCS-4 was subsequently introduced to allow
support for the larger Unicode codespace.  However, UCS-4 proved somewhat
impractical, partly because of its excessive storage requirements, and partly
because it is not backwards-compatible with existing UCS-2 data streams.
:p.The introduction of UTF-16 provided a solution to both of these problems.
Consequently, UTF-16 is commonly used for internal Unicode processing on many
of the platforms which originally used UCS-2 (although OS/2, as noted, continues
to use UCS-2 even today).
:p.All three of these encodings, however, share several other disadvantages&colon.
:ul.
:li.None of them is backwards compatible with single-byte ASCII data streams
(although conversion is normally quite easy).
:li.All of them require between two and four times as much storage space as
basic ASCII to encode the same characters.
:li.Data streams in any of these formats will contain embedded zero bytes,
which many APIs may interpret incorrectly as string termination characters.
:li.Each of these formats depends on reliable tracking of character boundaries
when parsing text.  A single lost or corrupted byte can compromise the integrity
of all the data that follows.
:eul.
:p.The UTF-8 encoding format eliminates all of these disadvantages.  However,
its encoding algorithm is non-trivial, and so overhead becomes an issue when
processing UTF-8 data.
:p.In general, UTF-8 is ideal for data storage and interchange, whereas UCS-2
and UTF-16 are preferred for internal processing.
:nt.UTF-8 and UTF-16 also share the property that no byte sequence representing
a valid character can possibly occur within a longer byte sequence representing
a different character; this is important when parsing or searching text.  Many
other variable-width encodings, including UTF-7 and most legacy multi-byte
codepages, do not share this property.:ent.

:p.
:p.:hp7.Naming conventions:ehp7.
:p.The original fixed-width UCS encoding formats (UCS-2 and UCS-4) were named
according to the convention :hp2.UCS-:ehp2.:hp3.x:ehp3., where :hp1.x:ehp1.
represents the fixed number of bytes used for each character.
:p.With the introduction of variable-width encodings (UTF-16, UTF-8 and UTF-7),
the naming convention for Unicode encodings was changed to
:hp2.UTF-:ehp2.:hp3.y:ehp3., where UTF stands for "Unicode Transformation
Format", and :hp1.y:ehp1. represents the minimum number of bits required to
represent a single character.
:p.UCS-4 was accordingly renamed UTF-32.  UCS-2 is now treated as a subset of
UTF-16; the two terms should not, however, be used interchangeably.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=20200.UniChars and character values
:p.Most OS/2 programming environments normally deal with single-byte character
values (such as the C :hp1.char:ehp1. type).  When dealing with Unicode text,
however, conversion between these values and double-byte UniChar characters
becomes an issue.

:p.A UniChar value can be assigned in a number of ways.  If you know the
UCS codepoint of the desired character, you can assign it directly as an
integer.  For example, an uppercase "A" has a codepoint of U+0041 (0x0041).
:xmp.
    UniChar uc = 0x0041;
:exmp.
:p.Alternatively, since most OS/2 C compilers define the :hp1.wchar_t:ehp1.
type as an unsigned short integer, it is possible to convert character values
into UniChar values (and vice versa) using C wide-character conventions.
:xmp.
    UniChar uc = L'A';
    UniChar *ucString = (UniChar *) L"Now is the time";

    printf("%lc - %ls\n", uc, ucString );
:exmp.
:p.However, this technique should be avoided when dealing with characters that
may fall outside the basic ASCII range.  This is because, for most standard
OS/2 codepages, only the ASCII characters map to the equivalent values within
the UCS codespace.

:p.The most comprehensive way to convert between single-byte and UniChar
characters is to use the ULS :link reftype=hd res=30000.codepage conversion
functions:elink.&colon.
:ul compact.
:li.:hp1.UniStrToUcs:ehp1. and :hp1.UniUconvToUcs:ehp1. will convert a character
string from any known codepage into a UniChar (UCS-2) string.
:li.:hp1.UniStrFromUcs:ehp1. and :hp1.UniUconvFromUcs:ehp1. will convert a
UniChar string into a character string encoded in the desired codepage.
:eul.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=20300.Handling Unicode text
:p.As described, OS/2 represents Unicode characters internally as fixed-width
two-byte values (UCS-2 encoding).  These are treated as integral values for
processing purposes.  However, issues arise when it comes time to interpret
them as literal characters.

:p.Consider the following string&colon.
:xmp.
    Hello world
:exmp.
:p.In standard ASCII, this would consist of the byte sequence&colon.
:xmp.
    48 65 6C 6C 6F 20 77 6F 72 6C 64
:exmp.
:p.When converted to a UniChar string, each character value now occupies
two bytes instead of one, resulting in the byte sequence&colon.
:xmp.
    00 48 00 65 00 6C 00 6C 00 6F 00 20 00 77 00 6F 00 72 00 6C 00 64
:exmp.
:p.Attempting to use such a value as a literal string will fail in many
cases&colon. most string-handling functions treat one byte as a single atomic
unit, and the embedded zero bytes will be seen (incorrectly) as string
terminators.

:p.As mentioned in the previous section, it is sometimes possible to use C
wide-character conventions to circumvent this problem, at least with those
APIs that support it.  Doing so, however, does not necessarily convert the
Unicode values into meaningful characters for the current codepage or vice
versa (certain compilers may support this, but many do not); so in practice
this technique can only be applied to basic ASCII text, which is generally
the same across different codepages.  In any case, not all functions accept
wide-character input, and neither do Presentation Manager controls.

:p.In general, character-based text processing falls into one of two
categories&colon.
:ul compact.
:li.:link reftype=hd res=20301.Input and output:elink..
:li.:link reftype=hd res=20302.String manipulation:elink..
:eul.

:p.ULS provides mechanisms for handling Unicode text in both contexts.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20301.Input and output of Unicode text
:p.In general, UniChar strings must be converted to another format before
output (including to STDOUT/STDERR, printer devices, or Presentation Manager
controls).

:p.There are two basic approaches to doing so&colon.
:ul.
:li.Convert the text to a Unicode encoding which is designed for output.  Two
such encodings are available&colon. UPF-8 and UTF-8.  These encodings allow
Unicode display under Presentation Manager in conjunction with the corresponding
PM codepage&colon.
:dl compact tsize=14.
:dt.Codepage 1207
:dd.UPF-8 (8-bit Unicode Processing Format).
:dt.Codepage 1208
:dd.UTF-8 (8-bit Unicode Transformation Format).
:edl.
:p.In general, codepage 1207 (UPF-8) results in faster rendering of PM controls;
however, not all control types properly support UPF-8 output.  In addition, UPF-8
should not be used for data interchange.
:p.Neither of these codepages is available to text sessions under OS/2 (as they
cannot be used as process codepages).
:p.Output of non-ASCII text using a Unicode codepage is only possible using a
Unicode-compatible font (which is indicated by the FM_TYPE_UNICODE flag in the
:hp1.fsType:ehp1. field of the GPI :hp2.FONTMETRICS:ehp2. data
structure).  Consequently, bitmap fonts are not supported for this purpose; nor
are Type 1 fonts if the standard OS/2 ATM font driver is used.

:li.Convert the text to a legacy codepage encoding.  This approach is required
for display in a text session (OS/2 window or full-screen), in which case the
text should be converted to the current process codepage.
:eul.

:p.If the text is being output to file (or sent over a network) for interchange
with others, the output encoding need not match the current codepage if the text
is not intended for local display.  It should, however, use an appropriate
encoding for display by the recipient.  UTF-8 (codepage 1208) is often used for
this purpose, particularly in HTML documents and electronic messages.

:p.Conversely, text input should be implemented to accept text in a
byte-oriented format, which can then be converted to UCS-2 for subsequent
processing.

:p.Conversion between UCS-2 and other codepages is performed using the ULS
:link reftype=hd res=30000.conversion functions:elink..

:p.
:p.:hp7.Direct Output of Unicode Text under GPI:ehp7.

:p.It is possible to output UniChar text without conversion (that is, as UCS-2
data streams) if direct GPI rendering is used.  This can be done by using a font
which supports Unicode (as noted above), and setting the :hp1.usCodePage:ehp1.
field of the :hp2.FATTRS:ehp2. structure to 1200.

:p.This is possible because the GPI text-drawing functions use byte arrays with
explicitly-specified length variables, and so do not attempt to interpret null
bytes as terminators.  (When setting the string length parameter, however,
remember that one UniChar equals two bytes.)

:p.This technique is not, however, supported for printer devices, which require
codepage output.  Unicode text should be converted to codepage 1207 before
printing.

:p.The following sample of code illustrates the technique.  (This code is
assumed to be called during WM_PAINT processing in the window procedure.)
:xmp.
    HPS         hps;          // presentation space
    RECTL       rcl,          // window area
                rclClip;      // clipping region for drawn text
    POINTL      ptl;          // current text position
    FATTRS      fontAttrs;    // current font attributes
    FONTMETRICS fm;           // current font metrics
    UniChar   * pUniText;     // UCS-2 text to draw


    hps = WinBeginPaint( hwnd, NULLHANDLE, NULLHANDLE );
    memset( &amp.fontAttrs, 0, sizeof(FATTRS) );

    // set the font attributes and create the logical font
    fontAttrs.usRecordLength = sizeof(FATTRS);
    fontAttrs.usCodePage     = 1200;
    fontAttrs.fsType         = FATTR_TYPE_MBCS;
    fontAttrs.fsFontUse      = FATTR_FONTUSE_NOMIX;
    strcpy( fontAttrs.szFacename, "Times New Roman MT 30");
    GpiCreateLogFont( hps, NULL, 1L, &amp.fontAttrs );
    GpiSetCharSet( hps, 1L );

    // paint the window background
    WinQueryWindowRect( hwnd, &amp.rcl );
    WinFillRect( hps, &amp.rcl, SYSCLR_WINDOW );

    // define a suitable margin to leave around the text
    GpiQueryFontMetrics( hps, sizeof(FONTMETRICS), &amp.fm );
    rclClip.xLeft   = rcl.xLeft + fm.lAveCharWidth;
    rclClip.yBottom = rcl.yBottom + fm.lXHeight;
    rclClip.xRight  = rcl.xRight - fm.lAveCharWidth;
    rclClip.yTop    = rcl.yTop - fm.lXHeight;

    // set the text colour and position
    GpiSetColor( hps, SYSCLR_WINDOWTEXT );
    ptl.x = rclClip.xLeft;
    ptl.y = rclClip.yTop - fm.lExternalLeading - fm.lMaxAscender;

    // (set the contents of pUniText)
    // ...

    GpiCharStringPosAt( hps, &amp.ptl, &amp.rclClip, CHS_CLIP,
                        UniStrlen(pUniText) * 2, (PCH) pUniText, NULL );

    // (perform any additional processing required)
    // ...

    WinEndPaint( hps );
:exmp.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20302.Unicode string manipulation
:p.Due to the existence of embedded zero bytes, most standard library functions
for text processing cannot be used with UniChar text.  For this reason, ULS
includes equivalents of several standard C library functions which operate on
UniChar strings and characters.

:p.Most of these correspond to the :hp1.str*:ehp1. functions of the standard
string library (plus the character functions :hp1.tolower:ehp1. and
:hp1.toupper:ehp1.).

.* :p.In addition, certain functions provide advanced transformation operations
.* which are locale-specific.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=20400.Character attributes
:p.ULS provides a number of functions for determining whether or not a Unicode
character has a specific :hp2.attribute:ehp2..

:p.A character attribute (or classification) describes categories to which a
character belongs.  These include both POSIX types like 'alpha' (alphabetic
character) and 'digit' (numeric digit), and extended Unicode types like
'ideograph' (CJK ideograph) and 'nonspacing' (non-spacing character).  There
are also attributes for describing text layout, and others indicating the
specific character sets to which a character belongs.

:p.See the :link reftype=hd res=20401.next section:elink. for a complete
list of attributes.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20401.Attributes names and identifiers
:p.Every character attribute defined by ULS has a symbolic :hp2.identifier:ehp2.
(referencing an integer value) associated with it.  Most attributes also have an
:hp2.attribute name:ehp2., which is a human-readable UniChar string.

:p.The :hp1.UniQueryAttr:ehp1. function may be used to obtain the integer value
of the identifier associated with a specified attribute name.

:p.Character attributes are grouped into several different categories.&colon.
:dl break=fit tsize=18.
:dt.Character type
:dd.These attributes indicate the type of character (lowercase, uppercase,
punctuation, whitespace, and so on).  Their names all start with a lowercase
alphabetic character.
:p.There are three groups of character type attributes.
:ul.
:li.POSIX (XPG/4) types.  Their symbolic identifiers all start with
:hp2.CT_:ehp2..
:li.Extended (Unicode) types.  Their symbolic identifiers all start with
:hp2.C3_:ehp2..
:li.Win32 compatibility types.  These attributes are equivalent to POSIX
types, and do not have names; their symbolic identifiers all start with
:hp2.C1_:ehp2..
:eul.

:dt.Character set
:dd.These attributes indicate character sets (linguistic or symbolic) to which
a character may belong.  These attributes all have names starting with an
underscore character ("_"); their symbolic identifiers all begin with
:hp2.CHS_:ehp2..

:dt.BiDi
:dd.These attributes are used by Unicode for determining character orientation
(writing direction).  Their names all start with a number-sign character ("#"),
and their symbolic identifiers start with :hp2.C2_:ehp2..

:dt.Codepage
:dd.These attributes are used to indicate a subset of common codepages within
the OS/2 Universal Glyph List for which the characters are valid.  They do not
have names; their symbolic identifiers all begin with :hp2.CCP_:ehp2..
:edl.

:p.All of the defined attributes are listed below.  Note that all attribute
names must be specified in lowercase.

:p.
:p.:hp7.Character type attributes:ehp7.
:p.The following are the standard POSIX attributes.  Multiple attributes may be
combined together, in calls to :hp1.UniQueryChar:ehp1., using bitwise OR
syntax.

:table cols='14 22 40' frame='box' rules='horiz'.
:row.
:c.NAME
:c.IDENTIFIER
:c.DESCRIPTION

:row.
:c.alnum
:c.CT_ALNUM
:c.Alphabetic and numeric characters

:row.
:c.alpha
:c.CT_ALPHA
:c.Letters and linguistic marks

:row.
:c.ascii
:c.CT_ASCII
:c.Standard ASCII character

:row.
:c.blank
:c.CT_BLANK
:c.Space and tab characters

:row.
:c.cntrl
:c.CT_CNTRL
:c.Control and format characters

:row.
:c.digit
:c.CT_DIGIT
:c.Digits 0 through 9

:row.
:c.graph
:c.CT_GRAPH
:c.All except controls and space

:row.
:c.lower
:c.CT_LOWER
:c.Lower case alphabetic character

:row.
:c.number
:c.CT_NUMBER
:c.Integral numbers between 0 and 9

:row.
:c.print
:c.CT_PRINT
:c.Everything except control characters

:row.
:c.punct
:c.CT_PUNCT
:c.Punctuation marks

:row.
:c.space
:c.CT_SPACE
:c.Whitespace and line-breaking characters

:row.
:c.symbol
:c.CT_SYMBOL
:c.Symbol

:row.
:c.upper
:c.CT_UPPER
:c.Upper case alphabetic character

:row.
:c.xdigit
:c.CT_XDIGIT
:c.Hexadecimal digits (0-9, a-f, A-F)
:etable.

:p.The following are the extended (Unicode) attributes.  Those without names
may only be used as bitmasks for the :hp2.UNICTYPE:ehp2. data structure and
the :hp1.UniQueryStringType:ehp1. function.

:table cols='14 22 40' frame='box' rules='horiz'.
:row.
:c.NAME
:c.IDENTIFIER
:c.DESCRIPTION

:row.
:c.
:c.C3_ALPHA
:c.Alphabetic character

:row.
:c.diacritic
:c.C3_DIACRITIC
:c.Diacritic mark

:row.
:c.fullwidth
:c.C3_FULLWIDTH
:c.Full-width variant

:row.
:c.halfwidth
:c.C3_HALFWIDTH
:c.Half-width variant

:row.
:c.hiragana
:c.C3_HIRAGANA
:c.Hiragana character

:row.
:c.ideograph
:c.C3_IDEOGRAPH
:c.Kanji/Han character

:row.
:c.kashida
:c.C3_KASHIDA
:c.Arabic tatweel (elongation character)

:row.
:c.katakana
:c.C3_KATAKANA
:c.Katakana character

:row.
:c.nonspacing
:c.C3_NONSPACING
:c.Non-spacing mark

:row.
:c.nsdiacritic
:c.C3_NSDIACRITIC
:c.Non-spacing diacritic

:row.
:c.nsvowel
:c.C3_NSVOWEL
:c.Non-spacing vowel

:row.
:c.
:c.C3_SYMBOL
:c.Symbol

:row.
:c.vowelmark
:c.C3_VOWELMARK
:c.Vowel mark
:etable.

:p.The following Win32 compatibility attributes are defined.

:table cols='14 22' frame='box' rules='horiz'.
:row.
:c.IDENTIFIER
:c.POSIX EQUIVALENT
:row.
:c.C1_UPPER
:c.CT_UPPER
:row.
:c.C1_LOWER
:c.CT_LOWER
:row.
:c.C1_DIGIT
:c.CT_DIGIT
:row.
:c.C1_SPACE
:c.CT_SPACE
:row.
:c.C1_PUNCT
:c.CT_PUNCT
:row.
:c.C1_CNTRL
:c.CT_CNTRL
:row.
:c.C1_BLANK
:c.CT_BLANK
:row.
:c.C1_XDIGIT
:c.CT_XDIGIT
:row.
:c.C1_ALPHA
:c.CT_ALPHA
:etable.

:p.:hp7.Character set attributes:ehp7.
:p.These attributes represent classes of character, and must be tested
individually.

:table cols='14 22 40' frame='box' rules='horiz'.
:row.
:c.NAME
:c.IDENTIFIER
:c.DESCRIPTION

:row.
:c._apl
:c.CHS_APL
:c.APL character

:row.
:c._arabic
:c.CHS_ARABIC
:c.Arabic character

:row.
:c._arrow
:c.CHS_ARROW
:c.Arrow character

:row.
:c._bengali
:c.CHS_BENGALI
:c.Bengali character

:row.
:c._bopomofo
:c.CHS_BOPOMOFO
:c.Bopomofo character

:row.
:c._box
:c.CHS_BOX
:c.Box or line drawing character

:row.
:c._currency
:c.CHS_CURRENCY
:c.Currency Symbol

:row.
:c._cyrillic
:c.CHS_CYRILLIC
:c.Cyrillic character

:row.
:c._dash
:c.CHS_DASH
:c.Dash character

:row.
:c._devanagari
:c.CHS_DEVANAGARI
:c.Devanagari character

:row.
:c._dingbat
:c.CHS_DINGBAT
:c.Dingbat

:row.
:c._fraction
:c.CHS_FRACTION
:c.Fraction value

:row.
:c._greek
:c.CHS_GREEK
:c.Greek character

:row.
:c._gujarati
:c.CHS_GUJARATI
:c.Gujarati character

:row.
:c._gurmukhi
:c.CHS_GURMUKHI
:c.Gurmukhi character

:row.
:c._hanguel
:c.CHS_HANGUEL
:c.Hangul Jamo character

:row.
:c._hebrew
:c.CHS_HEBREW
:c.Hebrew character

:row.
:c._hiragana
:c.CHS_HIRAGANA
:c.Hiragana character set

:row.
:c._katakana
:c.CHS_KATAKANA
:c.Katakana character set

:row.
:c._lao
:c.CHS_LAO
:c.Laotian character

:row.
:c._latin
:c.CHS_LATIN
:c.Latin character

:row.
:c._linesep
:c.CHS_LINESEP
:c.Line separator

:row.
:c._math
:c.CHS_MATH
:c.Math symbol

:row.
:c._punctstart
:c.CHS_PUNCTSTART
:c.Punctuation start

:row.
:c._punctend
:c.CHS_PUNCTEND
:c.Punctuation end

:row.
:c._tamil
:c.CHS_TAMIL
:c.Tamil character

:row.
:c._telegu
:c.CHS_TELEGU
:c.Telegu character

:row.
:c._thai
:c.CHS_THAI
:c.Thai character

:row.
:c._userdef
:c.CHS_USERDEF
:c.User defined character
:etable.


:p.:hp7.BiDi attributes:ehp7.
:p.These attributes represent classes of character, and must be tested
individually.
:table cols='14 22 40' frame='box' rules='horiz'.
:row.
:c.NAME
:c.IDENTIFIER
:c.DESCRIPTION

:row.
:c.#arabicnum
:c.C2_ARABICNUMBER
:c.Arabic numbers

:row.
:c.#blocksep
:c.C2_BLOCKSEPARATOR
:c.Block separator

:row.
:c.#commonsep
:c.C2_COMMONSEPARATOR
:c.Common separator

:row.
:c.#euronum
:c.C2_EUROPENUMBER
:c.European number

:row.
:c.#eurosep
:c.C2_EUROPESEPARATOR
:c.European separator

:row.
:c.#euroterm
:c.C2_EUROPETERMINATOR
:c.European terminator

:row.
:c.#left
:c.C2_LEFTTORIGHT
:c.Left to right text orientation

:row.
:c.#mirrored
:c.C2_MIRRORED
:c.Symmetrical text orientation

:row.
:c.#neutral
:c.C2_OTHERNEUTRAL
:c.Other neutral

:row.
:c.#right
:c.C2_RIGHTTOLEFT
:c.Right to left text orientation

:row.
:c.#whitespace
:c.C2_WHITESPACE
:c.Whitespace
:etable.


:p.:hp7.Codepage attributes:ehp7.
:p.These attributes are only used as bitmasks for the :hp2.UNICTYPE:ehp2. data
structure and the :hp1.UniQueryStringType:ehp1. function.
:table cols='14 22' frame='box' rules='horiz'.
:row.
:c.IDENTIFIER
:c.DESCRIPTION

:row.
:c.CCP_437
:c.US PC
:row.
:c.CCP_850
:c.Multilingual PC
:row.
:c.CCP_SYMB
:c.PostScript Symbol
:row.
:c.CCP_1252
:c.Windows Latin 1
:row.
:c.CCP_1250
:c.Windows Latin 2
:row.
:c.CCP_1251
:c.Windows Cyrillic
:row.
:c.CCP_1254
:c.Windows Turkish
:row.
:c.CCP_1257
:c.Windows Baltic
:etable.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20402.Localized attributes and the AttrObject type
:p.Character attributes are often dependent on locale.  The :hp2.AttrObject:ehp2.
data type is used to represent one or more attributes in a locale-specific
context.
:nt.Refer to the section on :link reftype=hd res=40000.localization:elink. for
information on using locales.:ent.

:p.An AttrObject is created using the :hp1.UniCreateAttrObject:ehp1.
function.  This function requires you to specify a locale using a
LocaleObject; you must also specify a UniChar string containing the desired
attribute names (separated by spaces).

:p.An AttrObject should be freed using :hp1.UniFreeAttrObject:ehp1. when it
is no longer needed.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20403.Querying character attributes
:p.The attributes of a specific UCS-2 character may be queried using either
locale-specific or locale-independent functions.

:p.
:p.:hp7.Locale-specific functions:ehp7.
:p.These functions query character attributes according to the rules defined
by a particular locale.

:p.The :hp1.UniQueryCharAttr:ehp1. function is used to determine whether a
character has the attribute(s) defined by the specified AttrObject.

:p.In addition, the standard POSIX attributes may be queried using dedicated
functions which are similar to the :hp1.is*:ehp1. functions of the C
character-type library, except that the attributes are localized according to
the specified LocaleObject.

:p.Finally, :hp1.UniScanForAttr:ehp1. function may be used to search for
characters within a UniChar string that match (either positively or negatively)
the attributes defined by the specified AttrObject.

:p.
:p.:hp7.Locale-independent functions:ehp7.
:p.These functions query character attributes independently of locale.

:p.The :hp1.UniQueryChar:ehp1. function is used to determine whether a
character has the character-type attribute(s) defined by the specified
identifier(s).  This function may only be used to check for the standard
POSIX or Win32 type attributes (CT_* or C1_*), which may be combined using
bitwise OR syntax.  An integer value of 0 or 1 is returned, depending on
whether or not the character posses the specified attribute(s).

:p.The :hp1.UniQueryStringType:ehp1. function returns an array of USHORT values
for the specified UniChar string, where each value in the array describes the
attributes of a single character according to the specified category&colon.
:dl compact tsize=18 break=fit.
:dt.CT_ITYPE
:dd.Return POSIX (XPG/4) character type attributes.  Test using :hp2.CT_*:ehp2.
identifiers.
:dt.CT_EXTENDED
:dd.Return extended (Unicode) charater type attributes.  Test using
:hp2.C3_*:ehp2. identifiers.
:dt.CT_CHARSET
:dd.Return character set attributes.  Test using :hp2.CHS_*:ehp2. identifiers.
:dt.CT_BIDI
:dd.Return BiDi (layout) attributes.  Test using :hp2.C2_*:ehp2. identifiers.
:dt.CT_CODEPAGE
:dd.Return codepage information.  Test using :hp2.CCP_*:ehp2. identifiers.
:dt.CT_INDEX
:dd.Return information used by the character type table (in conjunction with
:hp1.UniQueryCharTypeTable:ehp1.).
:dt.CT_CTYPE1
:dd.Return POSIX character type attributes in a Win32-compatible format.  Test
using :hp2.C1_*:ehp2. identifiers.
:dt.CT_CTYPE2
:dd.Return BiDi attributes in a Win32-compatible format.  Test using
:hp2.C2_*:ehp2. identifiers.
:dt.CT_CTYPE3
:dd.Return extended character type attributes in a Win32-compatible format.  Test
using :hp2.C3_*:ehp2. identifiers.
:edl.

:p.The :hp1.UniQueryCharType:ehp1. function queries all the attributes of a
specified UCS-2 character, and places the information in a UNICTYPE data
structure.  This structure contains information about character attributes in
all categories&colon.
:xmp.
typedef struct {
  USHORT    itype;      /* POSIX character types mask (CT_*/C1_*) */
  CHAR      bidi;       /* BiDi attributes mask (C2_*)            */
  CHAR      charset;    /* Character sets mask (CHS_*)            */
  USHORT    extend;     /* Extended character types mask (C3_*)   */
  USHORT    codepage;   /* Codepages mask (CCP_*)                 */
} UNICTYPE;
:exmp.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=20500.Transforming text
:p.ULS includes various functions for transforming strings of Unicode text in
locale-specific ways.  Available transformations include&colon.
:ul compact.
:li.Converting alphabetic text between upper- and lowercase.
:li.Converting diacritical text between composed and decomposed character forms.
:li.Converting Japanese phonetic text (Hiragana/Katakana) from one syllabary to
another.
:eul.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20501.Case conversion functions
:p.The functions :hp1.UniTransLower:ehp1. and :hp1.UniTransUpper:ehp1. may be
used to convert text to lowercase or uppercase, respectively, according to the
case-conversion rules for the specified locale.

:p.Both functions take as parameters a UniChar string to be converted, and a
LocaleObject to govern the conversion.

:nt.These two functions should not be confused with :hp1.UniStrlwr:ehp1. and
:hp1.UniStrupr:ehp1., which convert case according to generic
(locale-independent) case-conversion rules.:ent.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=20502.Transformations using the XformObject type
:p.The :hp2.XformObject:ehp2. type is used to control text transformations.

:p.The :hp1.UniCreateTransformObject:ehp1. creates an XformObject for the
specified transformation type, according to the rules of a particular locale.

:p.The transformation type is specified as a UniChar string; the standard
types are&colon.

:dl tsize=14 break=fit.
:dt.:hp1.lower:ehp1.
:dd.Transform so that all text is lowercase.  Characters without lowercase forms
(as defined by the locale) are left unchanged.
:dt.:hp1.upper:ehp1.
:dd.Transform so that all text is uppercase.  Characters without uppercase forms
(as defined by the locale) are left unchanged.
:dt.:hp1.compose:ehp1.
:dd.Transform so that all diacritical (e.g. accented) characters are represented
using fully-composed forms, meaning that a single code element represents the
combined character.
:dt.:hp1.decompose:ehp1.
:dd.Transform so that all diacritical characters are represented using decomposed
forms, meaning that separate code elements are used to represent the base
character and the diacritical mark(s).
:dt.:hp1.hiragana:ehp1.
:dd.Transform so that all Japanese phonetic characters use the Hiragana
character set.
:dt.:hp1.katakana:ehp1.
:dd.Transform so that all Japanese phonetic characters use the Katakana
character set.
:dt.:hp1.kana:ehp1.
:dd.Transform so that all Japanese phonetic characters use the half-width
Katakana character set.
:edl.
:p.Other transformation types may be defined by specific locales.

:p.An XformObject should be freed using using :hp1.UniFreeTransformObject:ehp1.
once it is no longer needed.


:p.
:p.:hp7.UniTransformStr:ehp7.
:p.The conversion is performed by calling the :hp1.UniTransformStr:ehp1.
function.  This function takes the following parameters&colon.
:ol.
:li.An XformObject which defines the transformation.
:li.The input buffer (an array of UniChar values representing the string to be
transformed).
:li.A pointer to an integer value indicating input length; this should be set
to the length of the input buffer text, or -1 if the text is null-terminated.
:li.The output buffer (an array of UniChar values which will contain the
transformed string).
:li.A pointer to an integer value indicating output length; this should
initially be set to the length of the ouput buffer.
:eol.
:p.If the function returns successfully, the transformed text is placed into the
output buffer.  The input length value will be modified to contain the number of
characters read from the input string, and the output length value will contain
the total number of characters placed in the output buffer.
:p.Note that the input and output string lengths will not necessarily be the
same, particularly when using the :hp1.compose:ehp1. and :hp1.decompose:ehp1.
transformation types.  You should ensure that the output buffer is large enough
to hold the transformed text.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=20900.Summary of text processing functions
:p.:hp7.Unicode versions of standard library functions:ehp7.
:p.ULS provides the following equivalents of various standard C library functions,
which operate on Unicode (UCS-2) text.
:dl compact break=fit tsize=30.
:dt.:hp1.UniStrcat:ehp1.
:dd.Unicode :hp1.strcat:ehp1. - concatenates strings.
:dt.:hp1.UniStrchr:ehp1.
:dd.Unicode :hp1.strchr:ehp1. - locates the first occurance of a character.
:dt.:hp1.UniStrcmp:ehp1.
:dd.Unicode :hp1.strcmp:ehp1. - compares strings.
:dt.:hp1.UniStrcmpi:ehp1.
:dd.Unicode :hp1.strcmpi:ehp1. - compares strings case-insensitively.
:dt.:hp1.UniStrcoll:ehp1.
:dd.Unicode :hp1.strcoll:ehp1. - compares strings according to the specified locale's collating rules.
:dt.:hp1.UniStrcpy:ehp1.
:dd.Unicode :hp1.strcpy:ehp1. - copies a string.
:dt.:hp1.UniStrcspn:ehp1.
:dd.Unicode :hp1.strcspn:ehp1. - locates the first matching character.
:dt.:hp1.UniStrfmon:ehp1.
:dd.Unicode :hp1.strfmon:ehp1. - converts a monetary value to a string.
:dt.:hp1.UniStrftime:ehp1.
:dd.Unicode :hp1.strftime:ehp1. - converts a date/time value to a string.
:dt.:hp1.UniStrlen:ehp1.
:dd.Unicode :hp1.strlen:ehp1. - determines string length.
:dt.:hp1.UniStrlwr:ehp1.
:dd.Unicode :hp1.strlwr:ehp1. - converts a string to lowercase using language-neutral case mapping.
:dt.:hp1.UniStrncat:ehp1.
:dd.Unicode :hp1.strncat:ehp1. - concatenates strings up to a maximum length.
:dt.:hp1.UniStrncmp:ehp1.
:dd.Unicode :hp1.strncmp:ehp1. - compares strings up to a maximum length.
:dt.:hp1.UniStrncmpi:ehp1.
:dd.Unicode :hp1.strnicmp:ehp1. - compares strings case-insensitively up to a maximum length.
:dt.:hp1.UniStrncpy:ehp1.
:dd.Unicode :hp1.strncpy:ehp1. - copies a string up to a maximum length.
:dt.:hp1.UniStrpbrk:ehp1.
:dd.Unicode :hp1.strpbrk:ehp1. - returns a pointer to the first matching character.
:dt.:hp1.UniStrptime:ehp1.
:dd.Unicode :hp1.strptime:ehp1. - converts a string to a date/time value.
:dt.:hp1.UniStrrchr:ehp1.
:dd.Unicode :hp1.strrchr:ehp1. - locates the last occurance of a character.
:dt.:hp1.UniStrspn:ehp1.
:dd.Unicode :hp1.strspn:ehp1. - locates the first unmatched character.
:dt.:hp1.UniStrstr:ehp1.
:dd.Unicode :hp1.strstr:ehp1. - returns a pointer to the first matching substring.
:dt.:hp1.UniStrtod:ehp1.
:dd.Unicode :hp1.strtod:ehp1. - converts a string to a double floating point value.
:dt.:hp1.UniStrtol:ehp1.
:dd.Unicode :hp1.strtol:ehp1. - converts a string to a long integer value.
:dt.:hp1.UniStrtok:ehp1.
:dd.Unicode :hp1.strtok:ehp1. - tokenizes a string.
:dt.:hp1.UniStrtoul:ehp1.
:dd.Unicode :hp1.strtoul:ehp1. - converts a string to an unsigned long integer value.
:dt.:hp1.UniStrupr:ehp1.
:dd.Unicode :hp1.strupr:ehp1. - converts a string to uppercase using language-neutral case mapping.
:dt.:hp1.UniStrxfrm:ehp1.
:dd.Unicode :hp1.strxfrm:ehp1. - transforms a string into collating weights for the specified locale.
:dt.:hp1.UniTolower:ehp1.
:dd.Unicode :hp1.tolower:ehp1. - converts a character to lowercase using language-neutral case mapping.
:dt.:hp1.UniToupper:ehp1.
:dd.Unicode :hp1.toupper:ehp1. - converts a character to uppercase using language-neutral case mapping.
:edl.

:p.
:p.:hp7.Locale-independent text classification functions:ehp7.
:p.The following functions are used to obtain information about the attributes
of Unicode text.
:dl compact break=fit tsize=30.
:dt.:hp1.UniQueryAttr:ehp1.
:dd.Obtains the attribute number associated with the specified attribute name.
:dt.:hp1.UniQueryChar:ehp1.
:dd.Queries character attributes according to the specified attribute number(s).
:dt.:hp1.UniQueryCharType:ehp1.
:dd.Queries a character's type attributes.
:dt.:hp1.UniQueryCharTypeTable:ehp1.
:dd.Queries the character type table.
:dt.:hp1.UniQueryNumericValue:ehp1.
:dd.Queries the numeric value associated with a character.
:dt.:hp1.UniQueryStringType:ehp1.
:dd.Queries the type attributes of each character in a string.
:edl.

:p.
:p.:hp7.Localized text classification functions:ehp7.
:p.These classification functions operate according to the rules of a specific
locale.  (Refer to the :link reftype=hd res=40000.localization:elink. section
for information on using locales.)
:dl compact break=fit tsize=30.
:dt.:hp1.UniCreateAttrObject:ehp1.
:dd.Creates an AttrObject used to query character attributes according to the
specified locale.
:dt.:hp1.UniFreeAttrObject:ehp1.
:dd.Frees an AttrObject allocated by :hp1.UniCreateAttrObject:ehp1..
:dt.:hp1.UniQueryAlnum:ehp1.
:dd.Queries a character's alphanumeric attribute according to the specified locale.
:dt.:hp1.UniQueryAlpha:ehp1.
:dd.Queries a characters alphabetic attribute according to the specified locale.
:dt.:hp1.UniQueryBlank:ehp1.
:dd.Queries a character's blank attribute according to the specified locale.
:dt.:hp1.UniQueryCharAttr:ehp1.
:dd.Queries character attributes according to the specified AttrObject.
:dt.:hp1.UniQueryCntrl:ehp1.
:dd.Queries a character's control attribute according to the specified locale.
:dt.:hp1.UniQueryDigit:ehp1.
:dd.Queries a character's (decimal) digit attribute according to the specified locale.
:dt.:hp1.UniQueryGraph:ehp1.
:dd.Queries a character's graphic attribute according to the specified locale.
:dt.:hp1.UniQueryLower:ehp1.
:dd.Queries a character's lowercase attribute according to the specified locale.
:dt.:hp1.UniQueryPrint:ehp1.
:dd.Queries a character's printable attribute according to the specified locale.
:dt.:hp1.UniQueryPunct:ehp1.
:dd.Queries a character's punctuation attribute according to the specified locale.
:dt.:hp1.UniQuerySpace:ehp1.
:dd.Queries a character's space attribute according to the specified locale.
:dt.:hp1.UniQueryUpper:ehp1.
:dd.Queries a character's uppercase attribute according to the specified locale.
:dt.:hp1.UniQueryXdigit:ehp1.
:dd.Queries a character's hexadecimal-digit attribute according to the specified locale.
:dt.:hp1.UniScanForAttr:ehp1.
:dd.Searches a string for characters matching the criteria specified by an AttrObject.
:edl.

:p.
:p.:hp7.Localized text transformation functions:ehp7.
:p.The following functions are used to transform Unicode strings according to
locale-specific rules.  (Refer to the :link reftype=hd res=40000.localization:elink.
section for information on using locales.)
:dl compact break=fit tsize=30.
:dt.:hp1.UniCreateTransformObject:ehp1.
:dd.Creates an XformObject used to perform string transformations according to the specified locale.
:dt.:hp1.UniFreeTransformObject:ehp1.
:dd.Frees an XformObject allocated by :hp1.UniCreateTransformObject:ehp1..
:dt.:hp1.UniTransformStr:ehp1.
:dd.Transforms a string according to the specified XformObject.
:dt.:hp1.UniTransLower:ehp1.
:dd.Transforms a string to lowercase according to the specified locale.
:dt.:hp1.UniTransUpper:ehp1.
:dd.Transforms a string to uppercase according to the specified locale.
:edl.


.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=40000.Localization
:p.Localization is based around the concept of :hp2.locales:ehp2..  A locale
is a set of conventions, usually associated with a particular country, language,
and/or culture, that specifies how information should be presented.

:p.These conventions include such things as&colon.
:ul compact.
:li.The country normally associated with the locale
:li.The language normally associated with the locale
:li.The default unit of currency
:li.The standard number format
:li.The standard time and date format
:li.The standard measurement system
:li.The default paper size
:li.The default codepage(s) used for character processing
:eul.

:p.You can use locale information to determine the behaviour of your
applications as appropriate for the current environment.  In addition, several
ULS functions take locales as input (for instance, to determine collation or
case conversion rules).

:p.There are two types of locale, both of which may be used in this way.

:ul.
:li.:hp2.System locales:ehp2. are standard locales defined by OS/2.  They are
constant, and cannot be modified.  System locales exist for every country and
language known to OS/2, as well as for abstract conventions like "Posix", "C",
or "UNIV".  They are typically used to obtain generic information about a
particular locale.

:li.:hp2.User locales:ehp2. are locale instances which can be customized by the
user.  Unlike system locales, user locales can freely be created, modified, or
deleted.  All locale objects which appear in the OS/2 Locale (Country Palette)
applet, including the default desktop locale, are user locales.  They are
normally derived from system locales, but may be tailored according to the
user's own preferences.
:eul.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=40100.Referencing locales
:p.The :hp2.LocaleObject:ehp2. data type acts as a logical instance of a
specific locale.  A LocaleObject parameter is passed to a number of functions
in order to identify the locale whose information is being requested.

:p.The :hp1.UniCreateLocaleObject:ehp1. function is used to initialize a
LocaleObject which refers to the specified locale.  The locale can be
identified in one of the following ways&colon.
:ul compact.
:li.A UniChar string representing the human-readable locale name (such as
"en_US", "de_DE_EURO", "posix", etc.).
:li.A :hp2.LocaleToken:ehp2., which is an integer value used as a unique
symbolic identifier for each locale.
:li.A NULL pointer will create a LocaleObject for the generic "univ" locale.
:li.An empty string (either :hp1.char:ehp1. or UniChar) creates a LocaleObject
based on the locale defined by current environment variables.  (See the
:link reftype=hd res=40101.next section:elink. for details.)
:eul.

:nt.Note that, in the event that a specified locale name applies to both a
user locale and a system locale, ULS will always give precedence to the user
locale.:ent.

:p.If the desired locale name and/or LocaleToken is unknown, one or the other
may be obtained in various ways&colon.
:ul compact.
:li.The :hp1.UniQueryLocaleList:ehp1. function can be used to query the locale
names of all system and/or user locales.
:li.The :hp1.UniMapCtryToLocale:ehp1. function returns the locale name
corresponding to the specified country code.
:li.If the locale name is known, :hp1.UniLocaleStrToToken:ehp1. can be used
to return the corresponding LocaleToken.
:li.If the LocaleToken is known, :hp1.UniLocaleTokenToStr:ehp1. can be used
to return the corresponding locale name.
:eul.

:p.In addition, once you have the LocaleObject for a locale, you can retrieve
its corresponding locale name or LocaleToken at any subsequent time using
:hp1.UniQueryLocaleObject:ehp1..

:p.A LocaleObject should be freed using :hp1.UniFreeLocaleObject:ehp1. once it
is no longer needed.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=40101.Locale environment variables
:p.When :hp1.UniCreateLocaleObject:ehp1. is passed an empty string in place of
a locale identifier, a LocaleObject is created for the locale defined by the
current environment.  This is determined by a combination of the following
environment variables&colon.
:dl compact break=fit tsize=20.
:dt.LC_ALL
:dd.All locale settings.
:dt.LC_COLLATE
:dd.Collation settings.
:dt.LC_CTYPE
:dd.Character attributes (such as classification and case conversion).
:dt.LC_MESSAGES
:dd.Language used for output.
:dt.LC_MONETARY
:dd.Currency notation format and symbols.
:dt.LC_NUMERIC
:dd.Standard number formats.
:dt.LC_TIME
:dd.Time and date formats.
:dt.LANG
:dd.All locale settings not otherwise defined.
:edl.

:p.Each of these variables may be set to the name of an existing locale, in
which case the corresponding conventions for that locale will be applied to the
new locale.

:p.The variable LC_ALL indicates all locale conventions, and has priority over
all other variables (consequently, if LC_ALL is defined, all of the other
variables will be disregarded).  Conversely, the variable LANG has the lowest
priority; it applies only to those locale conventions which have not been
specified by any of the LC_* variables.  Each of the other variables applies
only to a specific subset of locale conventions, and has equal priority with
the others.

:p.
:p.:hp7.Examples:ehp7.
:p.The following environment values will set all locale conventions to
"fr_CA"&colon.
:xmp.
    LANG=fr_CA
:exmp.

:p.The following environment values will set conventions for date/time format to
"en_UK", and all other locale conventions to "en_US"&colon.
:xmp.
    LANG=en_US
    LC_TIME=en_UK
:exmp.

:p.The following environment values will set conventions for currency format to
"en_NZ", conventions for number format to "en_UK", and all other locale
conventions to "en_AU"&colon.
:xmp.
    LANG=en_AU
    LC_MONETARY=en_NZ
    LC_NUMERIC=en_UK
:exmp.

:p.The following environment values will set all locale conventions to "ja_JP"
(the setting for LC_NUMERIC is ignored)&colon.
:xmp.
    LC_ALL=ja_JP
    LC_NUMERIC=en_US
:exmp.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=40200.Querying locale information
:p.Each locale consists of a fixed number of key-value pairs known as
:hp2.locale items:ehp2. which describe the conventions for that locale.  Each
locale item is referred to by an integer key of the type :hp2.LocaleItem:ehp2..
:xmp.
    typedef int LocaleItem;
:exmp.
:p.Every LocaleItem has a symbolic name of the form
:hp2.LOCI_:ehp2.:hp3.name:ehp3., where the first character of :hp1.name:ehp1.
is a prefix indicating the value type&colon.
:ul compact.
:li.A prefix of :hp1.s:ehp1., :hp1.j:ehp1., or :hp1.w:ehp1. indicates a string
value.
:li.A prefix of :hp1.i:ehp1. or :hp1.x:ehp1. indicates an integer value.
:eul.
:p.The :link reftype=hd res=40201.following section:elink. lists all of the
defined locale items.  The total number of locale items (136 as of the latest
version of ULS) is indicated by the :hp2.LOCI_iMaxItem:ehp2. item; this should
be the same for every locale.

:p.The value of any item can be queried using the :hp1.UniQueryLocaleItem:ehp1.
function, which writes the current value to a string variable.  In addition,
items with integer values (those with a name prefix of :hp1.i:ehp1. or
:hp1.x:ehp1.) can be queried using :hp1.UniQueryLocaleValue:ehp1., which writes
the current value to an integer variable.

:p.
:p.:hp7.The locale conventions structure:ehp7.
:p.The locale conventions structure (:hp2.struct UniLconv:ehp2. or
:hp2.UNILCONV:ehp2.) contains information about how numbers and currency values
are represented under a given locale.  This structure may be obtained for a
specified locale using :hp1.UniQueryLocaleInfo:ehp1..

:xmp.
typedef struct UniLconv {
  UniChar   *decimal_point;         /* non-monetary decimal point            */
  UniChar   *thousands_sep;         /* non-monetary thousands separator      */
  short     *grouping;              /* non-monetary size of grouping         */
  UniChar   *int_curr_symbol;       /* int'l currency symbol and separator   */
  UniChar   *currency_symbol;       /* local  currency symbol                */
  UniChar   *mon_decimal_point;     /* monetary decimal point                */
  UniChar   *mon_thousands_sep;     /* monetary thousands separator          */
  short     *mon_grouping;          /* monetary size of grouping             */
  UniChar   *positive_sign;         /* non-negative values sign              */
  UniChar   *negative_sign;         /* negative values sign                  */
  short     int_frac_digits;        /* no of fractional digits int currency  */
  short     frac_digits;            /* no of fractional digits loc currency  */
  short     p_cs_precedes;          /* nonneg curr sym 1-precedes,0-succeeds */
  short     p_sep_by_space;         /* nonneg curr sym 1-space,0-no space    */
  short     n_cs_precedes;          /* neg curr sym 1-precedes,0-succeeds    */
  short     n_sep_by_space;         /* neg curr sym 1-space 0-no space       */
  short     p_sign_posn;            /* positioning of nonneg monetary sign   */
  short     n_sign_posn;            /* positioning of negative monetary sign */
  short     os2_mondecpt;           /* os2 curr sym positioning              */
  UniChar   *debit_sign;            /* non-neg-valued monetary sym - "DB"    */
  UniChar   *credit_sign;           /* negative-valued monetary sym - "CR"   */
  UniChar   *left_parenthesis;      /* negative-valued monetary sym - "("    */
  UniChar   *right_parenthesis;     /* negative-valued monetary sym - ")"    */
} UNILCONV;
:exmp.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=40201.Locale items
:p.Every locale contains the following locale items.  These may be specified as
parameters to :hp1.UniQueryLocaleItem:ehp1., :hp1.UniQueryLocaleValue:ehp1.,
and :hp1.UniSetUserLocaleItem:ehp1..

:xmp.
#define LOCI_sDateTime            1  /* Date and time format string       */
#define LOCI_sShortDate           2  /* Short date format                 */
#define LOCI_sTimeFormat          3  /* Time format string                */
#define LOCI_s1159                4  /* AM string                         */
#define LOCI_s2359                5  /* PM string                         */
#define LOCI_sAbbrevDayName7      6  /* Abbreviation of day 7 (Sun)       */
#define LOCI_sAbbrevDayName1      7  /* Abbreviation of day 1 (Mon)       */
#define LOCI_sAbbrevDayName2      8  /* Abbreviation of day 2 (Tue)       */
#define LOCI_sAbbrevDayName3      9  /* Abbreviation of day 3 (Wed)       */
#define LOCI_sAbbrevDayName4     10  /* Abbreviation of day 4 (Thu)       */
#define LOCI_sAbbrevDayName5     11  /* Abbreviation of day 5 (Fri)       */
#define LOCI_sAbbrevDayName6     12  /* Abbreviation of day 6 (Sat)       */
#define LOCI_sDayName7           13  /* Name of day of week 7 (Sun)       */
#define LOCI_sDayName1           14  /* Name of day of week 1 (Mon)       */
#define LOCI_sDayName2           15  /* Name of day of week 2 (Tue)       */
#define LOCI_sDayName3           16  /* Name of day of week 3 (Wed)       */
#define LOCI_sDayName4           17  /* Name of day of week 4 (Thu)       */
#define LOCI_sDayName5           18  /* Name of day of week 5 (Fri)       */
#define LOCI_sDayName6           19  /* Name of day of week 6 (Sat)       */
#define LOCI_sAbbrevMonthName1   20  /* Abbreviation of month 1           */
#define LOCI_sAbbrevMonthName2   21  /* Abbreviation of month 2           */
#define LOCI_sAbbrevMonthName3   22  /* Abbreviation of month 3           */
#define LOCI_sAbbrevMonthName4   23  /* Abbreviation of month 4           */
#define LOCI_sAbbrevMonthName5   24  /* Abbreviation of month 5           */
#define LOCI_sAbbrevMonthName6   25  /* Abbreviation of month 6           */
#define LOCI_sAbbrevMonthName7   26  /* Abbreviation of month 7           */
#define LOCI_sAbbrevMonthName8   27  /* Abbreviation of month 8           */
#define LOCI_sAbbrevMonthName9   28  /* Abbreviation of month 9           */
#define LOCI_sAbbrevMonthName10  29  /* Abbreviation of month 10          */
#define LOCI_sAbbrevMonthName11  30  /* Abbreviation of month 11          */
#define LOCI_sAbbrevMonthName12  31  /* Abbreviation of month 12          */
#define LOCI_sMonthName1         32  /* Name of month 1                   */
#define LOCI_sMonthName2         33  /* Name of month 2                   */
#define LOCI_sMonthName3         34  /* Name of month 3                   */
#define LOCI_sMonthName4         35  /* Name of month 4                   */
#define LOCI_sMonthName5         36  /* Name of month 5                   */
#define LOCI_sMonthName6         37  /* Name of month 6                   */
#define LOCI_sMonthName7         38  /* Name of month 7                   */
#define LOCI_sMonthName8         39  /* Name of month 8                   */
#define LOCI_sMonthName9         40  /* Name of month 9                   */
#define LOCI_sMonthName10        41  /* Name of month 10                  */
#define LOCI_sMonthName11        42  /* Name of month 11                  */
#define LOCI_sMonthName12        43  /* Name of month 12                  */
#define LOCI_sDecimal            44  /* Decimal point                     */
#define LOCI_sThousand           45  /* Triad separator                   */
#define LOCI_sYesString          46  /* Yes string                        */
#define LOCI_sNoString           47  /* No string                         */
#define LOCI_sCurrency           48  /* Currency symbol                   */
#define LOCI_sCodeSet            49  /* Locale codeset                    */
#define LOCI_xLocaleToken        50  /* IBM Locale Token                  */
#define LOCI_xWinLocale          51  /* Win32 Locale ID                   */
#define LOCI_iLocaleResnum       52  /* Resource number for description   */
#define LOCI_sNativeDigits       53  /* String of native digits           */
#define LOCI_iMaxItem            54  /* Maximum item number               */
#define LOCI_sTimeMark           55  /* Time mark (am/pm) format          */
#define LOCI_sEra                56  /* Era definition                    */
#define LOCI_sAltShortDate       57  /* Alternate short date format string*/
#define LOCI_sAltDateTime        58  /* Alternate date and time format    */
#define LOCI_sAltTimeFormat      59  /* Alternate time format             */
#define LOCI_sAltDigits          60  /* XPG4 alternate digits             */
#define LOCI_sYesExpr            61  /* XPG4 Yes expression               */
#define LOCI_sNoExpr             62  /* XPG4 No expression                */
#define LOCI_sDate               63  /* Short date separator              */
#define LOCI_sTime               64  /* Time separator                    */
#define LOCI_sList               65  /* List separator                    */
#define LOCI_sMonDecimalSep      66  /* Monetary currency separator       */
#define LOCI_sMonThousandSep     67  /* Monetary triad separator          */
#define LOCI_sGrouping           68  /* Grouping of digits                */
#define LOCI_sMonGrouping        69  /* Monetary groupings                */
#define LOCI_iMeasure            70  /* Measurement (Metric, British)     */
#define LOCI_iPaper              71  /* Normal paper size                 */
#define LOCI_iDigits             72  /* Digits to right of decimal        */
#define LOCI_iTime               73  /* Clock format                      */
#define LOCI_iDate               74  /* Format of short date              */
#define LOCI_iCurrency           75  /* Format of currency                */
#define LOCI_iCurrDigits         76  /* Digits to right for currency      */
#define LOCI_iLzero              77  /* Leading zero used                 */
#define LOCI_iNegNumber          78  /* Format of negative number         */
#define LOCI_iLDate              79  /* Format of long date               */
#define LOCI_iCalendarType       80  /* Type of default calandar          */
#define LOCI_iFirstDayOfWeek     81  /* First day of week (0=Mon)         */
#define LOCI_iFirstWeekOfYear    82  /* First week of year                */
#define LOCI_iNegCurr            83  /* Format of negative currency       */
#define LOCI_iTLzero             84  /* Leading zero on time              */
#define LOCI_iTimePrefix         85  /* AM/PM preceeds time               */
#define LOCI_iOptionalCalendar   86  /* Alternate calandar type           */
#define LOCI_sIntlSymbol         87  /* International currency symbol     */
#define LOCI_sAbbrevLangName     88  /* Windows language abbreviation     */
#define LOCI_sCollate            89  /* Collation table                   */
#define LOCI_iUpperType          90  /* Upper case algorithm              */
#define LOCI_iUpperMissing       91  /* Action for missing upper case     */
#define LOCI_sPositiveSign       92  /* Positive sign                     */
#define LOCI_sNegativeSign       93  /* Negative sign                     */
#define LOCI_sLeftNegative       94  /* Left paren for negative           */
#define LOCI_sRightNegative      95  /* Right paren for negative          */
#define LOCI_sLongDate           96  /* Long date formatting string       */
#define LOCI_sAltLongDate        97  /* Alternate long date format string */
#define LOCI_sMonthName13        98  /* Name of month 13                  */
#define LOCI_sAbbrevMonthName13  99  /* Abbreviation of month 13          */
#define LOCI_sName              100  /* OS/2 locale name                  */
#define LOCI_sLanguageID        101  /* Abbreviation for language (ISO)   */
#define LOCI_sCountryID         102  /* Abbreviation for country (ISO)    */
#define LOCI_sEngLanguage       103  /* English name of Language          */
#define LOCI_sLanguage          104  /* Native name of language           */
#define LOCI_sEngCountry        105  /* English name of country           */
#define LOCI_sCountry           106  /* Localized country name            */
#define LOCI_sNativeCtryName    107  /* Name of country in native language*/
#define LOCI_iCountry           108  /* Country code                      */
#define LOCI_sISOCodepage       109  /* ISO codepage name                 */
#define LOCI_iAnsiCodepage      110  /* Windows codepage                  */
#define LOCI_iCodepage          111  /* OS/2 primary codepage             */
#define LOCI_iAltCodepage       112  /* OS/2 alternate codepage           */
#define LOCI_iMacCodepage       113  /* Mac codepage                      */
#define LOCI_iEbcdicCodepage    114  /* Ebcdic codepage                   */
#define LOCI_sOtherCodepages    115  /* Other ASCII codepages             */
#define LOCI_sSetCodepage       116  /* Codpage to set on activation      */
#define LOCI_sKeyboard          117  /* Primary keyboard name             */
#define LOCI_sAltKeyboard       118  /* Alternate keyboard name           */
#define LOCI_sSetKeyboard       119  /* Keyboard to set on activation     */
#define LOCI_sDebit             120  /* Debit string                      */
#define LOCI_sCredit            121  /* Credit string                     */
#define LOCI_sLatin1Locale      122  /* Locale for Latin 1 names          */
#define LOCI_wTimeFormat        123  /* Win32 Time format                 */
#define LOCI_wShortDate         124  /* Win32 Date format                 */
#define LOCI_wLongDate          125  /* Win32 Long date format            */
#define LOCI_jISO3CountryName   126  /* Java abbrev for country (ISO-3)   */
#define LOCI_jPercentPattern    127  /* Java percent pattern              */
#define LOCI_jPercentSign       128  /* Java percent symbol               */
#define LOCI_jExponent          129  /* Java exponential symbol           */
#define LOCI_jFullTimeFormat    130  /* Java full time format             */
#define LOCI_jLongTimeFormat    131  /* Java long time format             */
#define LOCI_jShortTimeFormat   132  /* Java short time format            */
#define LOCI_jFullDateFormat    133  /* Java full date format             */
#define LOCI_jMediumDateFormat  134  /* Java medium date format           */
#define LOCI_jDateTimePattern   135  /* Java date time format pattern     */
#define LOCI_jEraStrings        136  /* Java era strings                  */
:exmp.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=40300.User locales
:p.Unlike system locales, which represent standardized locale conventions, user
locales are used to represent locally-configured preferences.  Normally, user
locales are created and modified by the user through the OS/2 Locale applet
(also called the Country Palette).  However, ULS also provides APIs for
manipulating user locales programatically.

:p.A user locale is created using the :hp1.UniMakeUserLocale:ehp1. function.
This function creates a new locale by copying an existing one (which may be
either a system locale, or a previously-created user locale).  Both the new
and existing locales are specified by locale name.

:p.A user locale is deleted using the :hp1.UniDeleteUserLocale:ehp1. function;
again, the locale is specified by name.

:p.Specific items within a user locale may be modified using the
:hp1.UniSetUserLocaleItem:ehp1. function.  This function takes as parameters
the locale name, the locale item to be modified, the type of value being
specified (which may be a codepage string, a UniChar string, or an integer,
depending on the item type), and finally the new item value itself.

:p.
:p.:hp7.Saving user locale changes:ehp7.

:p.Changes made to user locales (creations, deletions, or modifications) do not
take effect beyond the current process until the :hp1.UniCompleteUserLocale:ehp1.
function is called.  This function writes the entire set of user locales (with
any outstanding changes) to disk.

:p.User locale data is saved in the file :hp2.USER.LOC:ehp2. under the directory
:hp2.\LANGUAGE\LOCALE:ehp2. on the OS/2 boot drive.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=40900.Summary of localization functions
:p.:hp7.Locale information functions:ehp7.
:p.The following functions are used to obtain information about locales (both
system and user).
:dl compact break=fit tsize=30.
:dt.:hp1.UniCreateLocaleObject:ehp1.
:dd.Creates a LocaleObject handle for an existing locale.
:dt.:hp1.UniFreeLocaleInfo:ehp1.
:dd.Frees memory allocated by :hp1.UniQueryLocaleInfo:ehp1.
:dt.:hp1.UniFreeLocaleObject:ehp1.
:dd.Frees a LocaleObject allocated by :hp1.UniCreateLocaleObject:ehp1..
:dt.:hp1.UniFreeMem:ehp1.
:dd.Frees memory allocated by :hp1.UniQueryLocaleObject:ehp1.,
:hp1.UniLocaleTokenToStr:ehp1., or :hp1.UniQueryLocaleItem:ehp1..
:dt.:hp1.UniLocaleStrToToken:ehp1.
:dd.Obtains the LocaleToken corresponding to the specified locale name.
:dt.:hp1.UniLocaleTokenToStr:ehp1.
:dd.Obtains the locale name corresponding to the specified LocaleToken.
:dt.:hp1.UniMapCtryToLocale:ehp1.
:dd.Obtains the locale name corresponding to the specified OS/2 country code.
:dt.:hp1.UniQueryCountryName:ehp1.
:dd.Translates a country name into the specified language.
:dt.:hp1.UniQueryLanguageName:ehp1.
:dd.Translates a language name into the specified language.
:dt.:hp1.UniQueryLocaleInfo:ehp1.
:dd.Queries the specified locale's currency- and number-format conventions.
:dt.:hp1.UniQueryLocaleItem:ehp1.
:dd.Queries the value of the specified item within a locale.
:dt.:hp1.UniQueryLocaleList:ehp1.
:dd.Generates a list of all locales (system, user, or both) defined on the system.
:dt.:hp1.UniQueryLocaleObject:ehp1.
:dd.Obtains the locale name or LocaleToken corresponding to the specified LocaleObject.
:dt.:hp1.UniQueryLocaleValue:ehp1.
:dd.Queries the value of the specified integral item within a locale.
:edl.

:p.
:p.:hp7.User locale functions:ehp7.
:p.The following functions are specific to the manipulation of user locales.
:dl compact break=fit tsize=30.
:dt.:hp1.UniCompleteUserLocale:ehp1.
:dd.Saves all changes made by :hp1.UniSetUserLocaleItem:ehp1.,
:hp1.UniMakeUserLocale:ehp1., and :hp1.UniDeleteUserLocale:ehp1..
:dt.:hp1.UniDeleteUserLocale:ehp1.
:dd.Deletes the specified user locale.
:dt.:hp1.UniMakeUserLocale:ehp1.
:dd.Creates a new user locale.
:dt.:hp1.UniSetUserLocaleItem:ehp1.
:dd.Modifies the specified item within a user locale.
:edl.


.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=30000.Codepage Conversion
:p.The ULS conversion functions allow character strings in any particular
codepage to be converted into equivalent strings in UCS-2 (UniChar) format, and
vice versa.

:p.This conversion is designed to preserve character integrity across different
encodings&colon. that is, the logical value of the character is preserved, even
though the encoded value used to represent it may be different.

:p.For example, the :hp2.&ea.:ehp2. character (lowercase e with acute accent)
has value 0x82 under codepage 850, but value U+00E9 under Unicode.  Therefore, a
conversion of this character from codepage 850 to UCS-2 would convert the input
byte sequence "0x82" to the output byte sequence "0x00E9".

:p.Take the following string&colon.
:xmp.    A caf&ea. in Reykjav&ia.k.
:exmp.
:p.Under codepage 850, this consists of the byte sequence (in
hexadecimal)&colon.
:xmp.    41 20 63 61 66 82 20 69 6E 20 52 65 79 6B 6A 61 76 A1 6B 2E:exmp.
:p.Converting this string to UCS-2 would result in the output byte
sequence&colon.
:xmp.
    00 41 00 20 00 63 00 61 00 66 00 E9 00 20 00 69 00 6E 00 20
    00 52 00 65 00 79 00 6B 00 6A 00 61 00 76 00 ED 00 6B 00 2E
:exmp.
:p.The conversion not only expands the single-byte codepage characters into
two-byte UniChar values, it also ensures that the output UniChars have the
correct byte values for the characters they are intended to represent.

:p.Conversion to and from UCS-2 is symmetric&colon. if a codepage string is
converted to UCS-2, and then subsequently back to the original codepage, the
final string and the original string should be identical.

:p.Strings may also be converted from one multi-byte codepage to another, by
converting to UCS-2 as an intermediate step.  That is, to convert a string
from codepage A to codepage B, one first converts the string from codepage A
to UCS-2; one then takes the resulting string and converts it in turn from
UCS-2 to codepage B.

:nt.The term "multi-byte codepage" is used throughout this chapter to refer
to non-UCS-2 codepages in general&colon. that is, codepages under which text
may be processed byte-by-byte, using the :hp1.char:ehp1. type (and which are
thus compatible with standard APIs).  These may correspond to SBCS, DBCS, or
variable-width encodings.  This is in contrast to the UCS-2 encoding, under
which the atomic character is a two-byte (UniChar) value.
:p.Similarly, "multi-byte text" refers to any text encoded for a non-UCS-2
codepage (whether SBCS, DBCS, or variable-width)&colon. in other words, text
that may be processed as :hp1.char:ehp1. values without loss of integrity.:ent.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=30100.Conversion objects
:p.Every conversion between UCS-2 and a specific codepage is controlled by an
instance of the :hp2.UconvObject:ehp2. type (called a "conversion object").

:p.The :hp1.UniCreateUconvObject:ehp1. function creates a UconvObject for the
specified codepage.  This codepage and the optional conversion attributes are
both defined through the :link reftype=hd res=30101.conversion
specifier:elink..

:p.The conversion attributes may also be modified subsequently, and in slightly
greater detail, using the :hp1.UniSetUconvObject:ehp1. function in conjunction
with the :hp2.uconv_attribute_t:ehp2. data structure.  Similarly, attributes may
be queried using :hp1.UniQueryUconvObject:ehp1..

:p.The UconvObject may be used to convert a string from the associated codepage
to UCS-2, or from UCS-2 to the associated codepage, depending on the function(s)
used.  See :link reftype=hd res=30200.performing conversions:elink. for details.

:p.Once it is no longer needed, a UconvObject should be freed using
:hp1.UniFreeUconvObject:ehp1..


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=30101.Conversion specifiers
:p.The codepage identifier passed to :hp1.UniCreateUconvObject:ehp1. is a
specially-formatted UniChar string called a :hp2.conversion specifier:ehp2..

:p.The conversion specifier consists of the codepage name, optionally followed
by an :hp2.@:ehp2. symbol and zero or more comma-separated :hp2.conversion
modifiers:ehp2..

:p.The codepage name normally takes the form :hp2.IBM-:ehp2.:hp3.cpnum:ehp3.,
where :hp1.cpnum:ehp1. is the integer number of the codepage (see
:link reftype=hd res=60000.Appendix B:elink. for a list).  This name can be
generated from the codepage number using the :hp1.UniMapCpToUcsCp:ehp1.
function.  If the codepage name is left blank, the current process codepage
will be used.  There are also constants defined in :hp2.uconv.h:ehp2. for
several :link reftype=fn refid=cpnames.common codepages:elink..

:p.The conversion modifiers affect the behaviour of conversions performed
using the UconvObject.  The following modifiers can be specified&colon.

:dl.
:dt.:hp1.map:ehp1.
:dd.Some byte values (specifically, the ASCII values 0x00 through 0x19 and
0x7F) may be used either as controls or to represent displayable glyphs,
depending on the context.  This modifier specifies how these characters should
be interpreted during conversion.
:p.The possible values are&colon.
:dl compact tsize=14.
:dt.map=data
:dd.Treat as control characters; leave the values unchanged in the output
text.  This is the default.
:dt.map=display
:dd.Treat as displayable glyphs; convert according to codepage like any
other character.
:dt.map=cdra
:dd.Treat as control codes; attempt to convert to the equivalent control
values in the output encoding (using standard IBM conversion rules).
:dt.map=crlf
:dd.Treat CR (carriage return) and LF (line feed) as control codes; treat all
others as displayable glyphs.
:edl.


:dt.:hp1.path:ehp1.
:dd.Specifies whether or not strings should be assumed to contain path
specifications when converting to or from DBCS codepages.
:p.Several Asian DBCS encodings which are otherwise ASCII-compatible replace
the ASCII backslash character (0x5C) with an Asian currency symbol (yen, yuan,
or won), and the ASCII tilde (0x7E) with an overline character. The purpose of
this parameter is to control how these character values should be treated when
converting to and from such codepages.
:p.This modifier is only respected when the :hp1.UniUconvFromUcs:ehp1. and
:hp1.UniUconvToUcs:ehp1. functions are used for the conversion.  When the
:hp1.UniStrFromUcs:ehp1. and :hp1.UniStrToUcs:ehp1. functions are used, the
behaviour is always that of :hp2.path=yes:ehp2..
:p.The possible values are&colon.
:dl tsize=14.
:dt.path=yes
:dd.Assume that strings may contain path names; convert 0x5C as the ASCII
backslash, and 0x7E as the ASCII tilde.  This has the following effects:
:ul compact.
:li.When converting from one of the affected DBCS codepages, byte value 0x5C
will be interpreted as the ASCII backslash, and 0x7E will be interpreted as the
ASCII tilde.
:li.When converting to one of the affected DBCS codepages, both the backslash
and the applicable Asian currency symbol will be converted to byte value 0x5C,
and both the tilde and the overline characters will be converted to byte value
0x7E.
:eul.
:p.This is the default.
:dt.path=no
:dd.Assumes that strings do not contain path data; convert 0x5C as the
single-byte Asian currency symbol, and 0x7E as the single-byte overline
character.  This has the following effects:
:ul compact.
:li.When converting from one of the affected DBCS codepages, byte value 0x5C
will be interpreted as the applicable Asian currency symbol, and byte value
0x7E will be interpreted as an overline character.
:li.When converting to one of the affected DBCS codepages, the applicable
Asian currency symbol will be converted to byte value 0x5C, and the overline
character will be converted to 0x7E.  However, the ASCII backslash and the ASCII
tilde will be both treated as nonexistent characters, and replaced by
substitution symbols.
:eul.
:edl.
:nt.All the character values affected by this setting (backslash, tilde,
overline, and the various currency symbols) refer to the single-byte
('halfwidth') forms only.  DBCS codepages may contain double-byte ('fullwidth')
forms of all these characters, which are not affected by the path setting.:ent.

:dt.:hp1.endian:ehp1.
:dd.Specifies the byte order (endian) used by UCS-2 strings.
:ul compact.
:li.When converting text to Unicode, this byte order will be used in generating
the UCS-2 output string.
:li.When converting text from Unicode, this byte order will be used for parsing
the UCS-2 input string.
:eul.
:p.The possible values are&colon.
:dl compact tsize=14.
:dt.endian=big
:dd.Use big-endian byte order.
:dt.endian=little
:dd.Use little-endian byte order.
:dt.endian=system
:dd.Use the system's native byte order.  This is the default.
:edl.
:p.Different byte orders may also be specified for conversions to and from
Unicode.  This is done using the format
:hp2.endian=:ehp2.:hp3.source:ehp3.:hp2.&colon.:ehp2.:hp3.target:ehp3., where
:hp1.source:ehp1. is the byte order (:hp2.big:ehp2., :hp2.little:ehp2., or
:hp2.system:ehp2.) to apply when converting from Unicode, and
:hp1.target:ehp1. is the byte order to apply when converting to Unicode.

:dt.:hp1.sub:ehp1.
:dd.Specifies whether character substitution is enabled.  Character substitution
means that, when converting text, any characters which do not exist in the
target codepage will be replaced by a generic "substitution"
character.
:p.This modifier only applies when the :hp1.UniUconvFromUcs:ehp1. and
:hp1.UniUconvToUcs:ehp1. functions are used for the conversion.  Substitution is
:hp2.always:ehp2. performed, regardless of this setting, when the
:hp1.UniStrFromUcs:ehp1. and :hp1.UniStrToUcs:ehp1. functions are used.
:p.The possible values are&colon.
:dl compact tsize=14.
:dt.sub=yes
:dd.Enable substitution when converting to or from Unicode.
:dt.sub=no
:dd.Disable all substitution.
:dt.sub=to-ucs
:dd.Enable substitution only when converting to Unicode (with
:hp1.UniUconvToUcs:ehp1.).
:dt.sub=from-ucs
:dd.Enable substitution only when converting from Unicode (with
:hp1.UniUconvFromUcs:ehp1.).  This is the default.
:edl.

:dt.:hp1.subchar:ehp1.
:dd.Specifies the byte value, within the target codepage, of the substitution
character to be used (if substitution is enabled), when converting from
Unicode.  The possible values are&colon.
:dl compact tsize=14.
:dt.subchar=\x:hp1.XX:ehp1.
:dd.:hp1.XX:ehp1. is the hexadecimal codepoint of the desired substitution
character within the target codepage.
:dt.subchar=\D:hp1.#:ehp1.
:dd.:hp1.#:ehp1. is the decimal codepoint of the desired substitution character
within the target codepage.
:edl.
:p.The default substitution character depends on the codepage.  For most
single-byte PC codepages it is 0x7F (&house.).  Several ISO-8859 codepages
such as 819, however, use 0x1A (the ASCII SUB control); this may cause
difficulties if displayed under Presentation Manager, as PM does not recognize
most ASCII control codes.
:nt.Remember to double backslash characters when using C/C++ string notation
(e.g. "subchar=\\x7F").:ent.

:dt.:hp1.subuni:ehp1.
:dd.Specifies the Unicode codepoint of the substitution character to be used
(if substitution is enabled), when converting to Unicode.  The possible values
are&colon.
:dl compact tsize=16.
:dt.subuni=\x:hp1.XX:ehp1.\x:hp1.YY:ehp1.
:dd.:hp1.XX:ehp1. and :hp1.YY:ehp1. are the high- and low-order hexadecimal byte
values, respectively, for the UCS codepoint of the desired substitution
character.
:dt.subuni=\x:hp1.XXXX:ehp1.
:dd.:hp1.XXXX:ehp1. is the hexadecimal UCS codepoint of the desired substitution
character.
:edl.
:p.The default substitution character is normally U+FFFD.
:nt.Remember to double backslash characters when using C/C++ string notation
(e.g. "subuni=\\xFFFD").:ent.

:edl.


:fn id=cpnames.
:p.The following codepage names are defined in :hp2.uconv.h:ehp2.&colon.
:xmp.
#define IBM_437    (UniChar *)L"IBM-437"   /* United States */
#define IBM_819    (UniChar *)L"IBM-819"   /* ISO Latin-1   */
#define ISO8859_1  (UniChar *)L"IBM-819"   /* ISO Latin-1   */
#define IBM_850    (UniChar *)L"IBM-850"   /* PC Latin-1    */
#define UCS_2      (UniChar *)L"IBM-1200"  /* UCS-2         */
#define UTF_8      (UniChar *)L"IBM-1208"  /* UTF-8         */
:exmp.
:efn.


.* Examples
.* Advanced attribute manipulation with uconv_attribute_t (including fine-tuned
.* control over control character mapping?


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=30200.Performing conversions
:p.ULS provides two sets of functions for converting text to or from UCS-2.
:p.Care should be taken as these two sets of functions differ slightly in
their parameters.  In particular, the :hp1.UniStr*Ucs:ehp1. functions specify
the output buffer first, followed by the input buffer.  Conversely, the
:hp1.UniUconv*Ucs:ehp1. functions specify the input buffer parameters first,
followed by the output buffer parameters.
:p.In addition, the parameters used to specify length behave somewhat
differently between the two sets of functions; see the
:link reftype=hd res=30201.next section:elink. for details.

:p.
:p.:hp7.UniStrToUcs and UniStrFromUcs:ehp7.
:p.These functions may be used to convert a string as a single operation, using
relatively straightforward syntax&colon.
:dl compact break=fit tsize=20.
:dt.:hp1.UniStrToUcs:ehp1.
:dd.Converts a multi-byte codepage string to a UCS-2 string.
:dt.:hp1.UniStrFromUcs:ehp1.
:dd.Converts a UCS-2 string to a multi-byte codepage string.
:edl.
:p.Each of these functions takes the following parameters&colon.
:ol.
:li.A UconvObject which defines the conversion.
:li.An output buffer, which must have been previously allocated.  This is a
UniChar string in the case of :hp1.UniStrToUcs:ehp1., or a normal string in
the case of :hp1.UniStrFromUcs:ehp1..
:li.An input buffer, which must have been previously allocated.  This is a
normal string in the case of :hp1.UniStrToUcs:ehp1., or a UniChar string in
the case of :hp1.UniStrFromUcs:ehp1..
:li.An integer value indicating the total length of the output buffer (in
bytes or UniChars, as applicable).  This must include one position for the
terminating null character.
:eol.
:p.If the function returns successfully, the converted string is placed in the
output buffer; the other parameters are unchanged.  If an error is encountered,
the conversion aborts and an error code is returned.  Character substitution is
always enabled, regardless of the UconvObject's attributes.

:p.
:p.:hp7.UniUconvToUcs and UniUconvFromUcs:ehp7.
:p.These functions are also used to convert strings, but with a greater level
of control and error recovery&colon.
:dl compact break=fit tsize=20.
:dt.:hp1.UniUconvToUcs:ehp1.
:dd.Converts a multi-byte codepage string to a UCS-2 string.
:dt.:hp1.UniUconvFromUcs:ehp1.
:dd.Converts a UCS-2 string to a multi-byte codepage string.
:edl.
:p.Each of these functions takes the following parameters&colon.
:ol.
:li.A UconvObject which defines the conversion (including whether or not
substitution is enabled).
:li.A pointer to a previously allocated input buffer&colon. this is a pointer to
a normal string in the case of :hp1.UniUconvToUcs:ehp1., or pointer to a UniChar
string in the case of :hp1.UniUconvFromUcs:ehp1..
:li.A pointer to a :hp1.size_t:ehp1. value indicating "input length"&colon. this
indicates the number of characters in the input buffer that must still be
converted.  This should initially be set to the length of the input buffer (not
including the terminating null character).
:li.A pointer to a previously allocated output buffer&colon. this is a pointer
to a UniChar string in the case of :hp1.UniUconvToUcs:ehp1., or pointer to a
normal string in the case of :hp1.UniUconvFromUcs:ehp1..
:li.A pointer to a :hp1.size_t:ehp1. value indicating "output length"&colon.
this indicates the number of empty character positions available in the output
buffer.  This length does not include the position allocated for a terminating
null character; it should initially be set to one less than the total length of
the output buffer (in bytes or UniChars, as applicable).
:li.A pointer to a :hp1.size_t:ehp1. value containing a substitution count.
:eol.
:p.When the function returns, the converted string is placed in the output
buffer, and the substitution count will contain the number of character
substitutions performed.  If an error is encountered during conversion, an error
code will be returned, and the parameters will be modified as follows&colon.
:ul.
:li.The output buffer will contain all characters successfully converted up to
the point when the error occured.
:li.The output length value will be decremented to indicate the number of
unassigned character positions still available in the output buffer.
:li.The pointer to the input buffer will be incremented to point to the first
character which has not yet been converted.
:li.The input length value will be decremented to indicate the number of
characters in the input buffer which have not yet been converted.
:li.The substitution count will contain the number of substitutions performed
up to the point when the error occured.
:eul.
:p.If the conversion completes with no errors, the input length value will be
set to 0 when the function returns.

:p.
:p.:hp7.Character substitution:ehp7.
:p.During conversion, particularly when converting from UCS-2 to another
codepage, it often occurs that a character is encountered in the input string
for which no equivalent character exists in the target codepage.  When this
happens, the unsupported character is normally replaced by a generic symbol
or "substitution character".  This process is known as :hp2.character
substitution:ehp2..

:p.Every codepage (as well as UCS-2) has its own default substitution character
which is normally used when converting text to that codepage.  For example,
under codepage 850 the default substitution character is 0x7F ("&house.").  The
default substitution character for UCS-2 is U+FFFD (the Unicode replacement
character).

:p.You may specify a different substitution character in the UconvObject,
either through the conversion specifier passed to
:hp1.UniCreateUconvObject:ehp1., or subsequently through
:hp1.UniSetUconvObject:ehp1..  The substitution character should be a
displayable glyph under the target codepage.

:p.When using the :hp1.UniUconvToUcs:ehp1. and :hp1.UniUconvFromUcs:ehp1.
functions, substitution may be disabled by setting the attributes of the
UconvObject accordingly.  However, if substitution is disabled, these functions
will return with an error condition (with the results described above) whenever
an unsupported character is encountered.


.* ............................................................................
:h3 x=left y=bottom width=100% height=100% res=30201.Conversion buffers
:p.This section contains some useful notes about managing the input and output
buffers used by the codepage conversion functions.

:p.:hp7.Determining Buffer Size:ehp7.
:p.When creating the output buffers for the conversion functions, care must be
taken to ensure that the buffer is large enough for the entire converted
output, plus a terminating null character.
:ul.
:li.When converting to Unicode, most multi-byte characters will map to a single
UniChar.  Therefore, it is generally safe to use an output buffer of the same
length as the input buffer.
:li.When converting from Unicode, you should take into account the possible
number of bytes per character which the specific target codepage allows.  While
single-byte (SBCS) codepages have a maximum of one byte per character,
multi-byte (MBCS) encodings may use up to four bytes to represent a single
character&colon.
:sl compact.
:li.- The common MBCS-PC codepages have a maximum of two bytes per character.
:li.- Most EUC-based codepages use up to three byte per character.  However,
EUC-TW (codepage 964) uses up to four bytes per character.
:li.- Both the UTF-8 (1208) and UPF-8 (1207) codepages use up to three bytes
per character.
:esl.
:p.Consequently, you should first identify the codepage(s) to which your buffer
may be used for conversion.  Identify the maximum number of bytes that could be
used for any of those codepages, and set your buffer size accordingly.
:p.If you wish to use your buffer for conversion to any arbitrary codepage, the
best practise is to use a buffer size of four times the length of the input
Unicode string (plus one byte for the terminating null).
:eul.

:p.
:p.:hp7.Using Length Parameters:ehp7.
:p.The two different sets of conversion functions differ in how they treat
their respective buffer length parameters.
:ul.
:li.With the :hp1.UniStr*Ucs:ehp1. functions, the last parameter indicates the
total size of the output buffer.  Consequently, it :hp2.must:ehp2. include
space for the terminating null character.
:li.By contrast, the length parameters used by the :hp1.UniUconv*Ucs:ehp1.
functions indicate numbers of character positions in the input and output
buffers. Somewhat confusingly, therefore, they should :hp2.not:ehp2. include
the terminating null character (and must therefore be set to one less than
the total length of the buffer).
:eul.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=30900.Summary of codepage conversion functions
:p.The following functions are used for codepage conversion.
:p.:hp7.Codepage conversion functions:ehp7.
:dl compact break=fit tsize=30.
:dt.:hp1.UniCreateUconvObject:ehp1.
:dd.Creates a UconvObject for conversions to/from the specified codepage.
:dt.:hp1.UniFreeUconvObject:ehp1.
:dd.Frees a UconvObject allocated by :hp1.UniCreateUconvObject:ehp1..
:dt.:hp1.UniMapCpToUcsCp:ehp1.
:dd.Converts a codepage number into a codepage specifier acceptable to :hp1.UniCreateUconvObject:ehp1..
:dt.:hp1.UniQueryUconvObject:ehp1.
:dd.Queries the attributes of a UconvObject.
:dt.:hp1.UniSetUconvObject:ehp1.
:dd.Modifies the attributes of a UconvObject.
:dt.:hp1.UniUconvFromUcs:ehp1.
:dd.Converts a UCS-2 string to a codepage string (with error recovery).
:dt.:hp1.UniUconvToUcs:ehp1.
:dd.Converts a codepage string to a UCS-2 string (with error recovery).
:dt.:hp1.UniStrFromUcs:ehp1.
:dd.Atomically converts a UCS-2 string to a codepage string.
:dt.:hp1.UniStrToUcs:ehp1.
:dd.Atomically converts a codepage string to a UCS-2 string.
:edl.


.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=50000.Appendix A&colon. The Universal Character Set
:p.The portion of the Unicode standard which describes the character codespace
is known as the :hp2.Universal Character Set:ehp2. (:hp2.UCS:ehp2.).  UCS is
also published separately by the International Organization for Standardization
as :hp2.ISO/IEC 10646:ehp2..
:p.The UCS codespace consists of 17 planes, each containing 65,536 characters.
These planes are listed below.  Only Plane 0 is directly supported by OS/2.

:dl break=none tsize=16.
:dt.Plane 0
:dd.(U+0000 - U+FFFF)
.br
:hp2.Basic Multilingual Plane:ehp2. (:hp2.BMP:ehp2.).  This plane covers most
modern writing systems.
:p.The first 256 codepoints of the BMP (U+0000 through U+00FF) map directly to
ISO-8859-1, the ISO Latin-1 encoding; this corresponds to OS/2 codepage 819.  On
OS/2 systems, this fact is principally useful with respect to the basic ASCII
character set (U+0020 through U+007F).

:dt.Plane 1
:dd.(U+10000 - U+1FFFF)
.br
:hp2.Supplementary Multilingual Plane:ehp2. (:hp2.SMP:ehp2.).  This plane is
primarily used for historical or academic scripts, as well as notational
characters which are rarely used in computing.

:dt.Plane 2
:dd.(U+20000 - U+2FFFF)
.br
:hp2.Supplementary Ideographic Plane:ehp2. (:hp2.SIP:ehp2.).  This plane is used
for rare (mostly historical) Chinese ideographic characters.

:dt.Planes 3 - 13
:dd.(U+30000 - U+DFFFF)
.br
Unassigned.

:dt.Plane 14
:dd.(U+E0000 - U+EFFFF)
.br
:hp2.Supplementary Special-Purpose Plane:ehp2. (:hp2.SSP:ehp2.).

:dt.Planes 15 - 16
:dd.(U+F0000 - U+10FFFF)
.br
:hp2.Supplementary Private Use Area:ehp2..
:edl.

:p.See the next two sections for a breakdown of the specific character sets
assigned to each value range.


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=50100.Supported characters sets in Unicode 2.1
:p.The Unicode support implemented in the conversion, transformation, and
classification functions of the ULS API roughly corresponds to version 2.1 of
the Unicode standard; this is limited, however, to the Basic Multilingual Plane.

:p.The range of characters which are actually capable of being displayed depends
on the Unicode-capable font(s) being used; such fonts may support more recent
versions of the Unicode standard, and thus will not necessarily be limited to
the character sets listed below.

:p.The character ranges defined in the Universal Character Set of Unicode 2.1
are listed below, according to their assigned blocks within the UCS codespace.

:xmp.
0000 - 007F  Basic Latin
0080 - 00FF  Latin-1 Supplement
0100 - 017F  Latin Extended-A
0180 - 024F  Latin Extended-B
0250 - 02AF  IPA Extensions
02B0 - 02FF  Spacing Modifier Letters
0300 - 036F  Combining Diacritical Marks
0370 - 03FF  Greek
0400 - 04FF  Cyrillic
0530 - 058F  Armenian
0590 - 05FF  Hebrew
0600 - 06FF  Arabic
0900 - 097F  Devanagari
0980 - 09FF  Bengali
0A00 - 0A7F  Gurmukhi
0A80 - 0AFF  Gujarati
0B00 - 0B7F  Oriya
0B80 - 0BFF  Tamil
0C00 - 0C7F  Telugu
0C80 - 0CFF  Kannada
0D00 - 0D7F  Malayalam
0E00 - 0E7F  Thai
0E80 - 0EFF  Lao
0F00 - 0FBF  Tibetan
10A0 - 10FF  Georgian
1100 - 11FF  Hangul Jamo
1E00 - 1EFF  Latin Extended Additional
1F00 - 1FFF  Greek Extended
2000 - 206F  General Punctuation
2070 - 209F  Superscripts and Subscripts
20A0 - 20CF  Currency Symbols
20D0 - 20FF  Combining Marks for Symbols
2100 - 214F  Letterlike Symbols
2150 - 218F  Number Forms
2190 - 21FF  Arrows
2200 - 22FF  Mathematical Operators
2300 - 23FF  Miscellaneous Technical
2400 - 243F  Control Pictures
2440 - 245F  Optical Character Recognition
2460 - 24FF  Enclosed Alphanumerics
2500 - 257F  Box Drawing
2580 - 259F  Block Elements
25A0 - 25FF  Geometric Shapes
2600 - 26FF  Miscellaneous Symbols
2700 - 27BF  Dingbats
3000 - 303F  CJK Symbols and Punctuation
3040 - 309F  Hiragana
30A0 - 30FF  Katakana
3100 - 312F  Bopomofo
3130 - 318F  Hangul Compatibility Jamo
3190 - 319F  Kanbun
3200 - 32FF  Enclosed CJK Letters and Months
3300 - 33FF  CJK Compatibility
4E00 - 9FFF  CJK Unified Ideographs
AC00 - D7A3  Hangul Syllables
D800 - DB7F  High Surrogates
DB80 - DBFF  High Private Use Surrogates
DC00 - DFFF  Low Surrogates
E000 - F8FF  Private Use
F900 - FAFF  CJK Compatibility Ideographs
FB00 - FB4F  Alphabetic Presentation Forms
FB50 - FDFF  Arabic Presentation Forms-A
FE20 - FE2F  Combining Half Marks
FE30 - FE4F  CJK Compatibility Forms
FE50 - FE6F  Small Form Variants
FE70 - FEFF  Arabic Presentation Forms-B
FF00 - FFEF  Halfwidth and Fullwidth Forms
FFF0 - FFFF  Specials
:exmp.

:p.This information is derived from the Unicode 2.0/2.1 documentation
(originally released in July 1996).


.* ----------------------------------------------------------------------------
:h2 x=left y=bottom width=100% height=100% res=50200.Supported characters sets in Unicode 5.0
:p.The Universal Character Set as defined in the Unicode 5.0 standard supports
the specific character sets listed below.

:p.Although the ULS functions themselves have not been updated for Unicode 5.0,
it may be possible to display characters listed here, or at least those within
the Basic Multilingual Plane (U+0000 - U+FFFF), when using modern Unicode fonts.

:xmp.
    0000 - 007F  Basic Latin
    0080 - 00FF  Latin-1 Supplement
    0100 - 017F  Latin Extended-A
    0180 - 024F  Latin Extended-B
    0250 - 02AF  IPA Extensions
    02B0 - 02FF  Spacing Modifier Letters
    0300 - 036F  Combining Diacritical Marks
    0370 - 03FF  Greek and Coptic
    0400 - 04FF  Cyrillic
    0500 - 052F  Cyrillic Supplement
    0530 - 058F  Armenian
    0590 - 05FF  Hebrew
    0600 - 06FF  Arabic
    0700 - 074F  Syriac
    0750 - 077F  Arabic Supplement
    0780 - 07BF  Thaana
    07C0 - 07FF  NKo
    0900 - 097F  Devanagari
    0980 - 09FF  Bengali
    0A00 - 0A7F  Gurmukhi
    0A80 - 0AFF  Gujarati
    0B00 - 0B7F  Oriya
    0B80 - 0BFF  Tamil
    0C00 - 0C7F  Telugu
    0C80 - 0CFF  Kannada
    0D00 - 0D7F  Malayalam
    0D80 - 0DFF  Sinhala
    0E00 - 0E7F  Thai
    0E80 - 0EFF  Lao
    0F00 - 0FFF  Tibetan
    1000 - 109F  Myanmar
    10A0 - 10FF  Georgian
    1100 - 11FF  Hangul Jamo
    1200 - 137F  Ethiopic
    1380 - 139F  Ethiopic Supplement
    13A0 - 13FF  Cherokee
    1400 - 167F  Unified Canadian Aboriginal Syllabics
    1680 - 169F  Ogham
    16A0 - 16FF  Runic
    1700 - 171F  Tagalog
    1720 - 173F  Hanunoo
    1740 - 175F  Buhid
    1760 - 177F  Tagbanwa
    1780 - 17FF  Khmer
    1800 - 18AF  Mongolian
    1900 - 194F  Limbu
    1950 - 197F  Tai Le
    1980 - 19DF  New Tai Lue
    19E0 - 19FF  Khmer Symbols
    1A00 - 1A1F  Buginese
    1B00 - 1B7F  Balinese
    1D00 - 1D7F  Phonetic Extensions
    1D80 - 1DBF  Phonetic Extensions Supplement
    1DC0 - 1DFF  Combining Diacritical Marks Supplement
    1E00 - 1EFF  Latin Extended Additional
    1F00 - 1FFF  Greek Extended
    2000 - 206F  General Punctuation
    2070 - 209F  Superscripts and Subscripts
    20A0 - 20CF  Currency Symbols
    20D0 - 20FF  Combining Diacritical Marks for Symbols
    2100 - 214F  Letterlike Symbols
    2150 - 218F  Number Forms
    2190 - 21FF  Arrows
    2200 - 22FF  Mathematical Operators
    2300 - 23FF  Miscellaneous Technical
    2400 - 243F  Control Pictures
    2440 - 245F  Optical Character Recognition
    2460 - 24FF  Enclosed Alphanumerics
    2500 - 257F  Box Drawing
    2580 - 259F  Block Elements
    25A0 - 25FF  Geometric Shapes
    2600 - 26FF  Miscellaneous Symbols
    2700 - 27BF  Dingbats
    27C0 - 27EF  Miscellaneous Mathematical Symbols-A
    27F0 - 27FF  Supplemental Arrows-A
    2800 - 28FF  Braille Patterns
    2900 - 297F  Supplemental Arrows-B
    2980 - 29FF  Miscellaneous Mathematical Symbols-B
    2A00 - 2AFF  Supplemental Mathematical Operators
    2B00 - 2BFF  Miscellaneous Symbols and Arrows
    2C00 - 2C5F  Glagolitic
    2C60 - 2C7F  Latin Extended-C
    2C80 - 2CFF  Coptic
    2D00 - 2D2F  Georgian Supplement
    2D30 - 2D7F  Tifinagh
    2D80 - 2DDF  Ethiopic Extended
    2E00 - 2E7F  Supplemental Punctuation
    2E80 - 2EFF  CJK Radicals Supplement
    2F00 - 2FDF  Kangxi Radicals
    2FF0 - 2FFF  Ideographic Description Characters
    3000 - 303F  CJK Symbols and Punctuation
    3040 - 309F  Hiragana
    30A0 - 30FF  Katakana
    3100 - 312F  Bopomofo
    3130 - 318F  Hangul Compatibility Jamo
    3190 - 319F  Kanbun
    31A0 - 31BF  Bopomofo Extended
    31C0 - 31EF  CJK Strokes
    31F0 - 31FF  Katakana Phonetic Extensions
    3200 - 32FF  Enclosed CJK Letters and Months
    3300 - 33FF  CJK Compatibility
    3400 - 4DBF  CJK Unified Ideographs Extension A
    4DC0 - 4DFF  Yijing Hexagram Symbols
    4E00 - 9FFF  CJK Unified Ideographs
    A000 - A48F  Yi Syllables
    A490 - A4CF  Yi Radicals
    A700 - A71F  Modifier Tone Letters
    A720 - A7FF  Latin Extended-D
    A800 - A82F  Syloti Nagri
    A840 - A87F  Phags-pa
    AC00 - D7AF  Hangul Syllables
    D800 - DB7F  High Surrogates
    DB80 - DBFF  High Private Use Surrogates
    DC00 - DFFF  Low Surrogates
    E000 - F8FF  Private Use Area
    F900 - FAFF  CJK Compatibility Ideographs
    FB00 - FB4F  Alphabetic Presentation Forms
    FB50 - FDFF  Arabic Presentation Forms-A
    FE00 - FE0F  Variation Selectors
    FE10 - FE1F  Vertical Forms
    FE20 - FE2F  Combining Half Marks
    FE30 - FE4F  CJK Compatibility Forms
    FE50 - FE6F  Small Form Variants
    FE70 - FEFF  Arabic Presentation Forms-B
    FF00 - FFEF  Halfwidth and Fullwidth Forms
    FFF0 - FFFF  Specials

  10000 - 1007F  Linear B Syllabary
  10080 - 100FF  Linear B Ideograms
  10100 - 1013F  Aegean Numbers
  10140 - 1018F  Ancient Greek Numbers
  10300 - 1032F  Old Italic
  10330 - 1034F  Gothic
  10380 - 1039F  Ugaritic
  103A0 - 103DF  Old Persian
  10400 - 1044F  Deseret
  10450 - 1047F  Shavian
  10480 - 104AF  Osmanya
  10800 - 1083F  Cypriot Syllabary
  10900 - 1091F  Phoenician
  10A00 - 10A5F  Kharoshthi
  12000 - 123FF  Cuneiform
  12400 - 1247F  Cuneiform Numbers and Punctuation
  1D000 - 1D0FF  Byzantine Musical Symbols
  1D100 - 1D1FF  Musical Symbols
  1D200 - 1D24F  Ancient Greek Musical Notation
  1D300 - 1D35F  Tai Xuan Jing Symbols
  1D360 - 1D37F  Counting Rod Numerals
  1D400 - 1D7FF  Mathematical Alphanumeric Symbols

  20000 - 2A6DF  CJK Unified Ideographs Extension B
  2F800 - 2FA1F  CJK Compatibility Ideographs Supplement

  E0000 - E007F  Tags
  E0100 - E01EF  Variation Selectors Supplement

  F0000 - FFFFF  Supplementary Private Use Area-A

100000 - 10FFFF  Supplementary Private Use Area-B
:exmp.

:p.This information is derived from the Unicode 5.0 documentation (released in
July 2006).  Development of the Unicode standard is ongoing, and new character
sets are added periodically; for complete information on the current version,
visit the Unicode Consortium's official web site at http&colon.//www.unicode.org.


.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=60000.Appendix B&colon. OS/2 Codepage Numbers
:p.The codepages listed below may be used in conjunction with the codepage
conversion functions.  Many of these are not available for use as system or PM
codepages, and will not be listed by the :hp1.WinQueryCpList:ehp1. function.

:p.While these lists are intended to be as comprehensive as possible, it is not
guaranteed that every possible OS/2 codepage will be shown.  In addition, not
all defined codepage aliases are included here.

:p.The codepage descriptions list the name of the applicable character set,
optionally followed by a comma and additional details about the specific
encoding standard used, optionally followed, in brackets, by the name of the
applicable country, region or application.

:p.Codepages prefixed with 'IBM' indicate encodings based on modifications of
the DOS Extended ASCII layout, and are not recommended for cross-platform
interchange.

:nt.Not all codepages may be available on all systems.:ent.


:p.
:p.:hp7.Unicode Codepages:ehp7.
:xmp.
1200  Unicode, UCS-2 (2-byte Universal Character Set encoding)
1207  Unicode, UPF-8 (8-bit Unicode Processing Format)
1208  Unicode, UTF-8 (8-bit Unicode Transformation Format)
:exmp.

:p.:hp7.ASCII-based (PC) Codepages:ehp7.
:xmp.
 367  ASCII, 7-bit
 437  DOS Extended ASCII (United States)
 813  ISO Greek, ISO-8859-7
 819  ISO Latin 1, ISO-8859-1
 850  IBM Latin 1, :link reftype=fn refid=ibm858.Modified:elink. (Multilingual)
 851  DOS Greek
 852  IBM Latin 2 (Eastern Europe)
 855  IBM Cyrillic
 856  DOS Hebrew
 857  IBM Latin 5 (Turkey)
 859  IBM Latin 9 (Multilingual)
 860  IBM Portuguese
 861  IBM Icelandic
 862  IBM Hebrew, :link reftype=fn refid=ibm867.Modified:elink. (Israel)
 863  IBM Canadian French
 864  IBM Arabic
 865  IBM Nordic
 866  IBM Russian
 868  IBM Urdu
 869  IBM Greek
 874  Thai, TIS-620/ISO-8859-11 Extended
 878  Internet Russian, KOI8-R
 891  Korean ASCII Extended + Half-Width Hangul, SBCS-PC
 895  Japanese JIS-Roman, 7-bit
 897  Japanese JISX0201-1976, SBCS-PC
 903  Simplified Chinese ASCII Extended, SBCS-PC (PRC)
 904  Traditional Chinese ASCII Extended, SBCS-PC (Taiwan)
 907  APL
 909  APL2 Extended
 910  APL2
 912  ISO Latin 2, ISO-8859-2
 913  ISO Latin 3, ISO-8859-3
 914  ISO Latin 4, ISO-8859-4
 915  ISO Cyrillic, ISO-8859-5
 916  ISO Hebrew, ISO-8859-8
 920  ISO Latin 5, ISO-8859-9
 921  ISO Latin 7, ISO-8859-13
 922  Estonian
 923  ISO Latin 9, ISO-8859-15 (Multilingual)
 932  Japanese, MBCS-PC/Shift-JIS  [aliased to 943]
 934  Korean, MBCS-PC legacy encoding  [aliased to 944]
 936  Simplified Chinese, MBCS-PC legacy encoding (PRC)  [aliased to 946]
 938  Traditional Chinese CNS11643 Extended, MBCS-PC (Taiwan)  [aliased to 948]
 942  Japanese JISX0201-1976 + JISX0208-1978 Extended, Shift-JIS
 943  Japanese JISX0201-1976 + JISX0208-1990 Windows31-J, Shift-JIS
 944  Korean SAA
 946  Simplified Chinese SAA (PRC)
 948  Traditional Chinese SAA (Taiwan)
 949  Korean KSC5601, :link reftype=fn refid=uhc.MBCS-PC/KS-Code:elink.
 950  Traditional Chinese Big-5, MBCS-PC/Big-5 (Taiwan)
 954  Japanese, EUC-JP
 964  Traditional Chinese, EUC-TW (Taiwan)
 970  Korean, EUC-KR
1004  Windows Latin 1 Extended
1006  Urdu
1008  Windows Arabic, Original
1041  Japanese JISX0201 Extended, SBCS-PC  [identical to 897]
1043  Traditional Chinese ASCII Extended, SBCS-PC  [identical to 904]
1051  HP Roman 8
1088  Korean ASCII Extended, SBCS-PC
1089  ISO Arabic, ISO-8859-6
1098  IBM Farsi
1114  ASCII + PC Symbols, 7-bit SBCS-PC (PRC &amp. Taiwan)
1115  Simplified Chinese ASCII Extended, SBCS-PC
1116  IBM Estonian
1117  IBM Latvian
1118  IBM Lithuanian
1119  IBM Lithuanian &amp. Russian
1124  Ukrainian, :link reftype=fn refid=ibm1124.Modified ISO Cyrillic:elink.
1125  IBM Ukrainian
1131  IBM Belarussian
1250  Windows Latin 2
1251  Windows Cyrillic
1252  Windows Latin 1
1253  Windows Greek
1254  Windows Turkish
1255  Windows Hebrew
1256  Windows Arabic
1257  Windows Latin 4
1275  Apple Latin 1
1276  Adobe PostScript Standard Encoding
1277  Adobe PostScript Latin 1 Encoding
1280  Apple Greek
1281  Apple Turkish
1282  Apple Central European
1283  Apple Cyrillic
1381  Simplified Chinese GB2312 Extended, MBCS-PC (PRC)
1383  Simplified Chinese, EUC-CN (PRC)
1386  Simplified Chinese GBK, MBCS-PC (PRC)
:exmp.

:p.:hp7.EBCDIC-based (mainframe) codepages:ehp7.
:xmp.
  37  United States/Canada, CECP
 273  Germany/Austria, CECP
 277  Denmark/Norway, CECP
 278  Finland/Sweden, CECP
 280  Italy, CECP
 282  Portugal, CECP
 284  Spain/Latin America, CECP
 285  United Kingdom, CECP
 290  Japanese Half-Width Katakana Extended, SBCS-Host
 293  APL
 297  France, CECP
 361  International - Publishing
 382  Austria/Germany - Publishing
 383  Belgium - Publishing
 385  Canadian French - Publishing
 386  Denmark/Norway - Publishing
 387  Finland/Sweden - Publishing
 388  France - Publishing
 389  Italy - Publishing
 391  Portugal - Publishing
 392  Spain/Philippines - Publishing
 393  Latin America - Publishing
 394  United Kingdom/Ireland - Publishing
 395  United States/Canada - Publishing
 420  Arabic Bilingual
 424  Hebrew (Israel)
 500  International
 833  Korean, SBCS-Host
 838  Thai
 870  Latin 2 (Eastern Europe)
 871  Iceland, CECP
 875  Greek
 918  Urdu
 924  Latin 9 (Multilingual)
 930  Japanese JISX0201-1976 + JISX208-1990 Extended, MBCS-Host
 931  Japanese Latin, MBCS-Host
 933  Korean KSC5601, MBCS-Host
 935  Simplified Chinese GB2312 Extended, MBCS-Host (PRC)
 937  Traditional Chinese CNS11643 Extended, MBCS-Host (Taiwan)
 939  Japanese Extended, MBCS-Host
1025  Cyrillic
1026  Latin 5 (Turkey)
1027  Japanese JISX0201-1976 Extended, SBCS-Host
1028  Hebrew - Publishing
1046  Arabic
1047  Open Systems
1097  Farsi
1112  Baltic
1122  Estonia
1123  Ukrainian
1140  United States/Canada, ECECP
1141  Germany, ECECP
1142  Denmark/Norway, ECECP
1143  Finland/Sweden, ECECP
1144  Italy, ECECP
1145  Spain/Latin America, ECECP
1146  UK/Ireland, ECECP
1147  France, ECECP
1148  Multilingual, ECECP
1149  Iceland, ECECP
:exmp.


:p.:hp7.Other Codepages:ehp7.
:xmp.
 259  Host Symbols - WP
 300  Japanese JISX0208-1990, DBCS-Host
 301  Japanese JISX0208-1990, DBCS-PC
 363  Host Symbols - Publishing
 829  Host Math Symbols - Publishing
 834  Korean KSC5601, DBCS-Host
 835  Traditional Chinese CNS11643 Extended, DBCS-Host (Taiwan)
 837  Simplified Chinese GB2312 Extended, DBCS-Host (PRC)
 896  Japanese Half-Width Katakana, 7-bit
 926  Korean, DBCS-PC legacy encoding  [aliased to 944]
 927  Traditional Chinese CNS11643 Extended, DBCS-PC  [aliased to 948]
 928  Simplified Chinese, DBCS-PC legacy encoding  [aliased to 946]
 941  Japanese JISX0208-1990 Extended, DBCS-PC/Shift-JIS  [aliased to 943]
 947  Traditional Chinese Big-5, DBCS-PC/Big-5  [aliased to 950]
 951  Korean KSC5601, DBCS-PC/KS-Code  [aliased to 949]
 952  Japanese JISX0208-1990, EUC encoded
 953  Japanese JISX0212-1990, EUC encoded
 955  Japanese JISX0208-1978, canonical
 971  Korean KSC5601, canonical
1038  Adobe PostScript Symbol Set
1092  Modified PC Symbols
1380  Simplified Chinese GB2312 Extended, DBCS-PC  [aliased to 1381]
:exmp.

:fn id=ibm858.
:p.The current implementation of codepage 850 has replaced the dotless i at
position 0xD5 with the Euro symbol.  (The modified codepage is officially
registered as IBM Coded Character Set ID number 858.)
:efn.

:fn id=ibm867.
:p.The current implementation of codepage 862 has replaced the Florin symbol at
position 0x9F with the New Shequel symbol, and the inverted exclamation mark at
position 0xAD with the Euro symbol.  (The modified codepage is officially
registered as IBM Coded Character Set ID number 867.)
:efn.

:fn id=uhc.
:p.The codepage 949 table (:hp2.IBM949:ehp2.) included by default with OS/2
supports only the KSC5601 EUC code set 1 plus IBM DBCS-PC extensions (0x9A**
to 0xA0**).
:p.A replacement codepage table is available through http&colon.//www.borgendale.com/uls.htm,
which updates codepage 949 to support the MS Unified Hangul Code ("Extended
Wansung") encoding.  This enables full support for the Johab character set.
:efn.

:fn id=ibm1124.
:p.Codepage 1124 is a modification of ISO-8859-5 (codepage 915) with the
Ukrainian GHE characters replacing GJE at positions 0xA3 and 0xF3.
:efn.


.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=80000.References
:p.The information described in this document was derived in part from the
resources listed below.

:p.Borgendale, Ken. :hp1.OS/2 Warp Internationalization:ehp1..  http&colon.//www.borgendale.com/uls.htm.

:p.International Business Machines Corporation.
.br
 -- :hp1.iSeries Globalization&colon. IBM Code Pages:ehp1..  http&colon.//www-03.ibm.com/servers/eserver/iseries/software/globalization/codepages.html
.br
 -- :hp1.OS/2 Warp Developer's Toolkit (version 4.52):ehp1..  1999.
.br
 -- :hp1.OS/2 Warp Server for e-business&colon. Keyboards and Codepages:ehp1..  1999.
.br
 -- :hp1.Unicode Functions (OS/2 Warp) - Second Edition:ehp1..  1997.

:p.Lunde, Ken.
.br
 -- :hp1.CJK.INF, Version 2.1:ehp1..  ftp&colon.//ftp.ora.com/pub/examples/nutshell/ujip/doc/cjk.inf.  1996.
.br
 -- :hp1.CJKV Information Processing&colon. Chinese, Japanese, Korean &amp. Vietnamese Computing:ehp1..  O'Reilly and Associates.  1999.

:p.Unicode Consortium, The.
.br
 -- Character mapping tables (various).  ftp&colon.//ftp.unicode.org/Public/MAPPINGS/
.br
 -- :hp1.The Unicode Standard, Version 4.0:ehp1..  2003.
.br
 -- :hp1.The Unicode Standard, Version 4.1:ehp1..  2005.
.br
 -- :hp1.The Unicode Standard, Version 5.0:ehp1..  2006.

:p.Wikipedia.
.br
 -- :hp1.Comparison of Unicode Encodings:ehp1..  http&colon.//en.wikipedia.org/wiki/Comparison_of_Unicode_encodings.
.br
 -- :hp1.Universal Character Set:ehp1..  http&colon.//en.wikipedia.org/wiki/Universal_Character_Set.
.br
 -- :hp1.UTF-16:ehp1..  http&colon.//en.wikipedia.org/wiki/UTF-16.
.br
 -- :hp1.UTF-32:ehp1..  http&colon.//en.wikipedia.org/wiki/UTF-32.
.br
 -- :hp1.UTF-7:ehp1..  http&colon.//en.wikipedia.org/wiki/UTF-7.
.br
 -- :hp1.UTF-8:ehp1..  http&colon.//en.wikipedia.org/wiki/UTF-8.


.******************************************************************************
:h1 x=left y=bottom width=100% height=100% res=90000.Notices
:p.:hp2.Universal Language Support (Unicode Functions) Programming Guide:ehp2.

:p.Version 1.5  (March 2011)
.br
(C) 2006, 2011 Alexander Taylor

:p.This document may be distributed without restriction, so long as it remains
in its original form and retains this copyright notice and list of trademarks.

:p.The following trademarks apply in the United States of America and other
countries.  Other names not listed here may be trademarks owned by other parties;
any omission should not be construed as a denial of such trademarks.
:ul.
:li.:hp2.IBM:ehp2., :hp2.OS/2:ehp2. and :hp2.Presentation Manager:ehp2. are
registered trademarks of International Business Machines Corporation.

:li.:hp2.ISO:ehp2. is a registered trademark of the International Organization for
Standardization.

:li.:hp2.Unicode:ehp2. is a trademark of the Unicode Consortium.

:li.:hp2.Windows:ehp2. is a registered trademark of Microsoft Corporation.
:eul.

:euserdoc.

